@@ -16,7 +16,9 @@ open Common
 open Debian
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 module Options = struct
   open OptParse
@@ -28,26 +30,28 @@ module Options = struct
   let options = OptParser.make ~description ~usage
   include BootstrapCommon.MakeOptions(struct let options = options end)
 
-  let addarchall = StdOpt.store_true ()
-  let noindep = StdOpt.store_false ()
+  let noarchall = StdOpt.store_true ()
   let allowsrcmismatch = StdOpt.store_true ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
 
-  add options ~group:prog_group ~long_name:"all" ~help:"also add source packages for Architecture:all packages" addarchall;
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
+  add options ~group:prog_group ~long_name:"noarchall" ~help:"do not add source packages for Architecture:all packages" noarchall;
   add options ~group:prog_group ~long_name:"allowsrcmismatch" ~help:("If a binary package is "^
     "without a source package but there is a source package of same name but "^
     "different version, match this binary package to that source package.") allowsrcmismatch;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly";"latest";"fg";"bg"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly";"latest";"fg";"bg";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options;;
+  
+  include StdOptions.OutputOptions;;
+  let default = List.filter (fun e -> not (List.mem e ["outdir"; "dot"])) StdOptions.OutputOptions.default_options in
+  StdOptions.OutputOptions.add_options ~default options;;
 
   include StdOptions.DistribOptions;;
-    let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+    let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options;;
 end
 
@@ -60,11 +64,10 @@ let main () =
   let buildarch = Option.get options.Debian.Debcudf.native in
   let hostarch = match options.Debian.Debcudf.host with None -> "" | Some s -> s in
   let foreignarchs = options.Debian.Debcudf.foreign in
-  let addarchall = OptParse.Opt.get Options.addarchall in
-  let noindep = OptParse.Opt.get Options.noindep in
+  let noarchall = OptParse.Opt.get Options.noarchall in
   let allowsrcmismatch = OptParse.Opt.get Options.allowsrcmismatch in
 
-  let binlist, (fgsrclist, bgsrclist), origsrclist = BootstrapCommon.parse_packages ~noindep Options.parse_cmdline buildarch hostarch foreignarchs posargs in
+  let binlist, (fgsrclist, bgsrclist), origsrclist = BootstrapCommon.parse_packages Options.parse_cmdline buildarch hostarch foreignarchs posargs in
   let srclist = fgsrclist @ bgsrclist in
   let tables = Debian.Debcudf.init_tables (srclist@binlist) in
   let sl = List.map (Debian.Debcudf.tocudf ~options tables) srclist in
@@ -78,10 +81,10 @@ let main () =
   in
 
   let bl =
-    if addarchall then
-      bl
-    else
+    if noarchall then
       List.filter BootstrapCommon.pkg_is_not_arch_all bl
+    else
+      bl
   in
 
   (* create a hashtable mapping package name and version tuples to
@@ -99,7 +102,10 @@ let main () =
    * packages are built *)
   let srcs = List.fold_left (fun acc bin ->
     let src = try BootstrapCommon.get_src_package ~allowmismatch:allowsrcmismatch universe bin
-    with Sources.NotfoundSrc -> failwith (Printf.sprintf "cannot find source for binary package %s" (CudfAdd.string_of_package bin)) in 
+      with Sources.NotfoundSrc ->
+        failwith (Printf.sprintf "cannot find source for binary package %s"
+                    (BootstrapCommon.string_of_package bin))
+    in
     CudfAdd.Cudf_set.add src acc
   ) CudfAdd.Cudf_set.empty bl in
 
@@ -117,7 +123,7 @@ let main () =
     let s = Hashtbl.find cudftosrc_table id in
     Debian.Printer.pp_source oc s;
     output_char oc '\n';
-  ) (BootstrapCommon.pkg_sort (CudfAdd.Cudf_set.elements srcs));
+  ) (BootstrapCommon.debcudf_sort (CudfAdd.Cudf_set.elements srcs));
 
   close_out oc;
 ;;
