@@ -16,7 +16,9 @@ open Common
 open Debian
 open Algo
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 let progressbar_build_graph = Util.Progress.create "build_graph"
 let progressbar_build_graph_bfs = Util.Progress.create "build_graph_bfs"
@@ -56,7 +58,16 @@ let default_edge = { depend = BuildsFrom (ref IntSet.empty); annotation = [] }
 
 type et = [`Loop | `NoLoop | `StrongDep ]
 
-let compare_edge e1 e2 = compare e1 e2
+(* we cannot just use the polymorphic compare here because that one checks
+ * for structural equality and the Set data structure is structurally
+ * different depending on the order in which elements were added to the set.
+ * Thus instead, use Set.compare *)
+let compare_edge e1 e2 =
+  match e1, e2 with
+  | { depend = BuildDep }, { depend = BuildDep } -> 0
+  | { depend = BuildsFrom s1 }, {depend = BuildsFrom s2} -> IntSet.compare !s1 !s2
+  | { depend = BuildDep }, { depend = BuildsFrom _ } -> 1
+  | { depend = BuildsFrom _ }, { depend = BuildDep } -> -1
 
 module PkgV = struct
   type t = Unique.t
@@ -77,67 +88,118 @@ module G = Graph.Imperative.Digraph.ConcreteBidirectionalLabeled(PkgV)(PkgE)
 module VertexSet = Set.Make(G.V)
 module EdgeSet = Set.Make(G.E)
 
-(* create a buildgraph containing all source packages in "sl" according to
- * partitioning scheme given by "partitionofsrc", connect binary packages to
- * source packages according to "bin2src" and omit connections for binary
- * packages that qualify as "available" *)
-let graph_helper available bin2src partitionofsrc univ sl =
+(* given a list of tuples where the first element is a source package and the
+ * second element is a partitioning of its installation set between its direct
+ * dependencies, connect binary packages to source packages according to
+ * "bin2src" and omit connections of binary packages to source packages for
+ * binary packages that qualify as "available" *)
+let graph_helper ?(num_cores=1) available bin2src univ partitionofsrc sl =
   let g = G.create () in
-  List.iter (fun srcpkg ->
-    Util.Progress.progress progressbar_build_graph;
-    let src = Unique.create (SrcPkg (CudfAdd.vartoint univ srcpkg)) in
-    let _, partitions = partitionofsrc srcpkg in
-    List.iter (fun (pid,is) ->
-        let instset = Unique.create (InstSet (pid,is)) in
-        let label = { depend = BuildDep; annotation = [] } in
-        let edge = (src,label,instset) in
-        G.add_edge_e g edge;
-        IntSet.iter (fun pid ->
-          let binpkg = CudfAdd.inttovar univ pid in
-          if not (available binpkg) then begin
-            let srcpkgdep = bin2src binpkg in
-            let dst = Unique.create (SrcPkg (CudfAdd.vartoint univ srcpkgdep)) in
-            try match G.E.label (G.find_edge g instset dst) with
-              |{ depend = BuildsFrom s } -> s := IntSet.add pid !s
-              |_ -> assert false 
-            with Not_found ->  begin
-              let label = { depend = BuildsFrom (ref (IntSet.singleton pid)); annotation = [] } in
-              let edge = (instset,label,dst) in
-              G.add_edge_e g edge
-            end
-          end
-        ) is
-    ) partitions;
-  ) sl;
-  g
+  let seen = Hashtbl.create (List.length sl) in
+  let rec aux = function
+    | [] -> g
+    | todo -> begin
+        (* generate dependency sets outside of graph generation because the
+         * graph is imperative and we must not executed code with side-effects
+         * inside a parallel block *)
+        let instsets = Parmap.parmap ~ncores:num_cores (fun srcpkg ->
+            Util.Progress.progress progressbar_build_graph;
+            let _, partitions = partitionofsrc srcpkg in
+            let srcpkgid = CudfAdd.vartoint univ srcpkg in
+            (srcpkgid, partitions)
+          ) (Parmap.L todo) in
+        (* we must not do this iteration in the block above because side
+         * effects from parallel blocks are discarded *)
+        List.iter (fun (srcpkgid,_) -> Hashtbl.add seen srcpkgid ()) instsets;
+        (* go over all source packages and their partitions installation sets
+         * and create a new list of source packages to work on if the new
+         * source package has not been seen yet *)
+        let todo = List.fold_left (fun acc (srcpkgid, partitions) ->
+            begin if not (Hashtbl.mem seen srcpkgid) then fatal "something went wrong" end;
+            let src = Unique.create (SrcPkg srcpkgid) in
+            (* we already add the source vertex here explicitly even though
+             * this vertex is normally added together with the edges to its
+             * InstSet vertices. This is so that when creating a graph starting
+             * from a single source package and that source package has
+             * unsatisfiable dependencies that source package will still be
+             * part of the output (with no outgoing edges) *)
+            G.add_vertex g src;
+            List.fold_left (fun acc (pid,is) ->
+                let instset = Unique.create (InstSet (pid,is)) in
+                let label = { depend = BuildDep; annotation = [] } in
+                let edge = (src,label,instset) in
+                G.add_edge_e g edge;
+                (* Only connect the InstSet vertex to the source packages
+                 * building the binary packages in "is" and not "pid".
+                 * "pid" has to be explicitly part of "is" if a connection
+                 * from this InstSet vertex to a SrcPkg should be made.
+                 * This is important to not include non-strong direct
+                 * dependencies in a strong dependency graph. *)
+                IntSet.fold (fun pid acc ->
+                    let binpkg = CudfAdd.inttovar univ pid in
+                    if not (available binpkg) then begin
+                      let srcpkgdep = bin2src binpkg in
+                      let srcpkgdepid = CudfAdd.vartoint univ srcpkgdep in
+                      let dst = Unique.create (SrcPkg srcpkgdepid) in
+                      begin try begin match G.E.label (G.find_edge g instset dst) with
+                        |{ depend = BuildsFrom s } -> s := IntSet.add pid !s
+                        |_ -> assert false
+                      end with Not_found ->  begin
+                          let label = { depend = BuildsFrom (ref (IntSet.singleton pid)); annotation = [] } in
+                          let edge = (instset,label,dst) in
+                          G.add_edge_e g edge
+                        end;
+                      end;
+                      if Hashtbl.mem seen srcpkgdepid then
+                        acc
+                      else begin
+                        Hashtbl.add seen srcpkgdepid ();
+                        srcpkgdep::acc
+                      end
+                    end else acc
+                  ) is acc
+              ) acc partitions
+          ) [] instsets
+        in
+        aux todo
+      end
+  in
+  aux sl
 ;;
 
 let get_src_package ?(allowmismatch=false) univ pkg = try
     BootstrapCommon.get_src_package ~allowmismatch univ pkg
   with Sources.NotfoundSrc ->
-    failwith (Printf.sprintf "can't find source package for binary package %s" (CudfAdd.string_of_package pkg))
+    failwith (Printf.sprintf "can't find source package for binary package %s"
+                (BootstrapCommon.string_of_package pkg))
 ;;
 
 (* create and time a normal buildgraph by calculating installation sets *)
-let dist_graph ?(available=(fun _ -> true)) ?(allowmismatch=false) custom_is_ht univ sl =
+let dist_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(opt=false) ?(num_cores=1) custom_is_ht univ sl =
   Util.Timer.start timer_build_graph;
   Util.Progress.set_total progressbar_build_graph (List.length sl);
 
   let bin2src = get_src_package ~allowmismatch univ in
-  let pool = Depsolver_int.init_pool_univ ~global_constraints:true univ in
+  let pool = Depsolver_int.init_pool_univ ~global_constraints univ in
 
-  let partitionofsrc = BootstrapCommon.compute_dependency_sets custom_is_ht pool univ in
+  let opt_partition_cache = Hashtbl.create 10000 in
+  let partitionofsrc = if opt then
+      BootstrapCommon.compute_dependency_sets_opt ~global_constraints ~partition:true ~available opt_partition_cache pool univ
+    else
+      BootstrapCommon.compute_dependency_sets ~global_constraints ~partition:true custom_is_ht pool univ
+  in
 
-  Util.Timer.stop timer_build_graph (graph_helper available bin2src partitionofsrc univ sl)
+  Util.Timer.stop timer_build_graph (graph_helper ~num_cores available bin2src univ partitionofsrc sl)
 ;;
 
 (* create and time a buildgraph by calculating installation closures *)
-let closure_graph ?(available=(fun _ -> true)) ?(allowmismatch=false) univ sl =
+let closure_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(num_cores=1) univ sl =
   Util.Timer.start timer_build_graph;
   Util.Progress.set_total progressbar_build_graph (List.length sl);
 
   let bin2src = get_src_package ~allowmismatch univ in
-  let pool = Depsolver_int.init_pool_univ ~global_constraints:true univ in
+  let pool = Depsolver_int.init_pool_univ ~global_constraints univ in
+  let globalid = Cudf.universe_size univ in
 
   let to_set l = List.fold_right IntSet.add l IntSet.empty in
   (* rule: connect everything to everything else *)
@@ -145,26 +207,49 @@ let closure_graph ?(available=(fun _ -> true)) ?(allowmismatch=false) univ sl =
     let l = List.fold_left (fun acc vpkglist ->
       let disj = CudfAdd.resolve_vpkgs_int univ vpkglist in
       List.fold_left (fun acc2 pid ->
-        let dc = Depsolver_int.dependency_closure_cache pool [pid] in
-        let dcs = to_set dc in
-        (pid, dcs)::acc2
+          let dc = Depsolver_int.dependency_closure_cache pool [pid; globalid] in
+          let dcs = IntSet.remove globalid (to_set dc) in
+          (pid, dcs)::acc2
       ) acc disj
     ) [] src.Cudf.depends in
     IntSet.empty, l
   in
 
-  Util.Timer.stop timer_build_graph (graph_helper available bin2src partitionofsrc univ sl)
+  Util.Timer.stop timer_build_graph (graph_helper ~num_cores available bin2src univ partitionofsrc sl)
 ;;
 
 (* create and time a normal buildgraph by calculating strong dependencies *)
-let strong_graph ?(available=(fun _ -> true)) ?(allowmismatch=false) univ sl =
+let strong_graph ?(global_constraints=true) ?(available=(fun _ -> true)) ?(allowmismatch=false) ?(num_cores=1) univ sl =
   Util.Timer.start timer_build_graph;
   Util.Progress.set_total progressbar_build_graph (List.length sl);
 
   let bin2src = get_src_package ~allowmismatch univ in
-  let pool = Depsolver_int.init_pool_univ ~global_constraints:true univ in
+  let pool = Depsolver_int.init_pool_univ ~global_constraints univ in
+  let globalid = Cudf.universe_size univ in
 
   let sdg = Strongdeps.strongdeps_univ univ in
+  (* connect all binary packages to the Essential:yes packages *)
+  if global_constraints then begin
+    (* get all binary packages that are Essential:yes *)
+    let essential = Cudf.get_packages
+        ~filter:(fun pkg ->
+            BootstrapCommon.debtype_of_cudfpkg pkg = `BinPkg
+            && BootstrapCommon.debessential_of_cudfpkg pkg)
+        univ
+    in
+    (* calculate the strong dependency graph of all Essential:yes packages and
+     * add all its edges to the original graph. This is done because some
+     * Essential:yes packages are never explicitly depended upon so their
+     * vertices will otherwise not show up in the graph *)
+    Defaultgraphs.PackageGraph.G.iter_edges
+      (Defaultgraphs.PackageGraph.G.add_edge sdg)
+      (Strongdeps.strongdeps univ essential);
+    (* connect all vertices in the original graph to all Essential:yes packages *)
+    Defaultgraphs.PackageGraph.G.iter_vertex
+      (fun v -> List.iter
+          (Defaultgraphs.PackageGraph.add_edge ~transitive:true sdg v)
+          essential) sdg
+  end;
 
   let tointset l = List.fold_left (fun acc e -> IntSet.add (CudfAdd.vartoint univ e) acc) IntSet.empty l in
   let to_set l = List.fold_right IntSet.add l IntSet.empty in
@@ -173,30 +258,37 @@ let strong_graph ?(available=(fun _ -> true)) ?(allowmismatch=false) univ sl =
   (* if the dependency closure of any direct dependency has a non-empty
    * intersection with the set of strong dependencies, add a connection. *)
   let partitionofsrc src =
-    let strongdeps = Defaultgraphs.PackageGraph.succ_list sdg src in
-    let strongdepss = tointset strongdeps in
-    let l = List.fold_left (fun acc vpkglist ->
-      let disj = CudfAdd.resolve_vpkgs_int univ vpkglist in
-      List.fold_left (fun acc2 pid ->
-        let dc = Depsolver_int.dependency_closure_cache pool [pid] in
-        let dcs = to_set dc in
-        let inters = IntSet.inter strongdepss dcs in
-        if not(IntSet.is_empty inters) then (pid, inters)::acc2 else acc2
-      ) acc disj
-    ) [] src.Cudf.depends in
-    IntSet.empty, l
+    (* the package in question might not be part of the strong dependency
+     * graph if it had unsatisfiable dependencies *)
+    if Defaultgraphs.PackageGraph.G.mem_vertex sdg src then
+      let strongdeps = Defaultgraphs.PackageGraph.succ_list sdg src in
+      let strongdepss = tointset strongdeps in
+      let l = List.fold_left (fun acc vpkglist ->
+          let disj = CudfAdd.resolve_vpkgs_int univ vpkglist in
+          List.fold_left (fun acc2 pid ->
+              let dc = Depsolver_int.dependency_closure_cache pool [pid; globalid] in
+              let dcs = to_set dc in
+              let inters = IntSet.inter strongdepss dcs in
+              if not(IntSet.is_empty inters) then (pid, inters)::acc2 else acc2
+            ) acc disj
+        ) [] src.Cudf.depends in
+      IntSet.empty, l
+    else begin
+      warning "package %s cannot be installed" (BootstrapCommon.string_of_package src);
+      IntSet.empty, []
+    end
   in
 
-  Util.Timer.stop timer_build_graph (graph_helper available bin2src partitionofsrc univ sl)
+  Util.Timer.stop timer_build_graph (graph_helper ~num_cores available bin2src univ partitionofsrc sl)
 ;;
 
 (* Build the dependency source graph bfs from a root node *)
-let dist_graph_bfs ?(maxdepth=max_int) ?(available=(fun _ -> true)) ?(allowmismatch=false) custom_is_ht univ rootlist =
+let dist_graph_bfs ?(global_constraints=true) ?(maxdepth=max_int) ?(available=(fun _ -> true)) ?(allowmismatch=false) custom_is_ht univ rootlist =
   Util.Timer.start timer_build_graph_bfs;
   Util.Progress.set_total progressbar_build_graph_bfs (Cudf.universe_size univ);
 
   let g = G.create () in
-  let pool = Depsolver_int.init_pool_univ ~global_constraints:true univ in
+  let pool = Depsolver_int.init_pool_univ ~global_constraints univ in
 
   let bin2src = get_src_package ~allowmismatch univ in
 
@@ -213,7 +305,7 @@ let dist_graph_bfs ?(maxdepth=max_int) ?(available=(fun _ -> true)) ?(allowmisma
       Util.Progress.progress progressbar_build_graph_bfs;
       Hashtbl.add visited src ();
       (* if the package cannot be compiled the List.iter will not happen *)
-      let _, partitions = BootstrapCommon.compute_dependency_sets custom_is_ht pool univ srcpkg in
+      let _, partitions = BootstrapCommon.compute_dependency_sets ~global_constraints ~partition:true custom_is_ht pool univ srcpkg in
       List.iter (fun (pid,is) ->
           (* let open Cudf in Cudf_printer.pp_package stdout (CudfAdd.inttovar univ pid); *)
           let instset = Unique.create (InstSet (pid,is)) in
@@ -249,7 +341,7 @@ let dist_graph_bfs ?(maxdepth=max_int) ?(available=(fun _ -> true)) ?(allowmisma
   Util.Timer.stop timer_build_graph_bfs g
 ;;
 
-let from_ic universe ic =
+let from_ic universe native_arch ic =
   let getstr n l =
    try match List.assoc n l with
      | GraphmlReader.String s -> s
@@ -263,49 +355,33 @@ let from_ic universe ic =
    with Not_found -> failwith (Printf.sprintf "cannot find key %s in list" n)
   in
 
-  (* this hashtable cuts about 1 second of runtime on a full graph for debian
-   * sid with 16k vert and 200k edges *)
-  let stringtointmap = Hashtbl.create 97 in
-
-  let stringtoint s =
-    match Hashtbl.find_option stringtointmap s with
-   | Some i -> i
-   | None -> begin
-      let name,version = String.split s " " in
-      let name = CudfAdd.encode name in
-      (* extract 1.0.0 from "(= 1.0.0)" *)
-      let version = String.sub version 3 ((String.length version) - 4) in
-      let pkgs = List.filter (fun p ->
-        CudfAdd.string_of_version p = version
-      ) (Cudf.lookup_packages universe name) in
-      let pkg = match pkgs with
-       | [] -> failwith (Printf.sprintf "cannot find %s (=%s)" name version)
-       | [p] -> p
-       | _ -> failwith (Printf.sprintf "more than one match for %s (=%s)" name version)
-      in
-      let i = CudfAdd.vartoint universe pkg in
-      Hashtbl.add stringtointmap s i;
-      i
-    end
-  in
-
   let node l =
     match getstr "kind" l with
      | "SrcPkg" -> begin
-       let name = CudfAdd.encode (getstr "name" l) in
+       let cudfname = CudfAdd.encode (getstr "cudfname" l) in
        let cudfversion = getint "cudfversion" l in
-       let srcpkg = try Cudf.lookup_package universe (name,cudfversion)
-       with Not_found -> failwith (Printf.sprintf "cannot find cudf package %s (= %d)" name cudfversion) in
+       let srcpkg = try
+           Cudf.lookup_package universe (cudfname,cudfversion)
+         with Not_found ->
+           failwith (Printf.sprintf "cannot find cudf package %s (= %d)"
+                       cudfname cudfversion)
+       in
        Unique.create (SrcPkg (CudfAdd.vartoint universe srcpkg))
      end
      | "InstSet" -> begin
-       let name = CudfAdd.encode (getstr "name" l) in
+       let cudfname = CudfAdd.encode (getstr "cudfname" l) in
        let cudfversion = getint "cudfversion" l in
-       let pkg = try Cudf.lookup_package universe (name,cudfversion)
-       with Not_found -> failwith (Printf.sprintf "cannot find cudf package %s (= %d)" name cudfversion) in
+       let pkg = try
+           Cudf.lookup_package universe (cudfname,cudfversion)
+         with Not_found ->
+           failwith (Printf.sprintf "cannot find cudf package %s (= %d)"
+                       cudfname cudfversion)
+       in
        let pid = CudfAdd.vartoint universe pkg in
-       let binaries = String.nsplit (getstr "binaries" l) "," in
-       let is = List.fold_right (fun s -> IntSet.add (stringtoint s)) binaries IntSet.empty in
+       let binaries = try getstr "binaries" l
+         with Not_found -> failwith "cannot find mandatory InstSet vertex attribute \"binaries\""
+       in
+       let is = BootstrapCommon.parse_debian_pkgstring universe native_arch binaries in
        Unique.create (InstSet (pid,is))
      end
      | _ -> failwith "invalid node kind"
@@ -315,8 +391,10 @@ let from_ic universe ic =
      | "builddep" ->
        {depend = BuildDep; annotation = []}
      | "buildsfrom" ->
-       let binaries = String.nsplit (getstr "binaries" l) "," in
-       let is = List.fold_right (fun s -> IntSet.add (stringtoint s)) binaries IntSet.empty in
+       let binaries = try getstr "binaries" l
+         with Not_found -> failwith "cannot find mandatory buildsfrom edge attribute \"binaries\""
+       in
+       let is = BootstrapCommon.parse_debian_pkgstring universe native_arch binaries in
        {depend = BuildsFrom (ref is); annotation = []}
      | _ -> failwith "invalid edge kind"
   in
@@ -360,6 +438,7 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
        "version","string",None;
        "binaries","string",None;
        "cudfversion","int",None;
+       "cudfname","string",None;
        "architecture","string",None;
        "priority","string",None;
        "type","string",None;
@@ -386,28 +465,25 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
     match vertex with
     |SrcPkg id->
       let pkg = CudfAdd.inttovar U.univ id in
-      let name = CudfAdd.decode pkg.Cudf.package in
-      let version = CudfAdd.string_of_version pkg in
-
       let kind = ("kind",string_of_vertex_kind vertex) in
-
-      let name = ("name",name) in
-      let version = ("version",version) in
-
+      let cudfname = ("cudfname", CudfAdd.decode pkg.Cudf.package) in
       let prop =
-        (* don't do anything if key is "version" and translate "cudfversion" to
-         * "version" *)
+        (* store cudf property "version" in vertex property "cudfversion"
+         * store cudf property "number" in vertex property "version"
+         * for all other vertex properties, take the cudf property directly *)
         List.filter_map (fun (key,_,_) ->
-          if key = "version" then None else begin
-            let k = if key = "cudfversion" then "version" else key in
+            let k =
+              if key = "cudfversion" then "version"
+              else if key = "version" then "number"
+              else key
+            in
             try
               let value = Cudf.lookup_package_property pkg k in
               Some(key,value)
             with Not_found -> None
-          end
         ) vertex_properties
       in
-      let prop = kind :: name :: version :: prop in
+      let prop = kind :: cudfname :: prop in
       (* only set those attributes which are not empty *)
       List.filter_map (fun (k,v) ->
         if v = "" then None else Some(k,v)
@@ -417,28 +493,30 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
         let bl =
           List.map (fun pid ->
             let pkg = CudfAdd.inttovar U.univ pid in
-            CudfAdd.string_of_package pkg
+            BootstrapCommon.string_of_package pkg
           ) (IntSet.elements is)
         in
         ("binaries",String.concat "," bl)
       in
       let pkg = CudfAdd.inttovar U.univ pid in
-      let name = CudfAdd.decode pkg.Cudf.package in
-      let version = CudfAdd.string_of_version pkg in
       let kind = ("kind",string_of_vertex_kind vertex) in
-      let name = ("name",name) in
-      let version = ("version",version) in
+      let cudfname = ("cudfname", CudfAdd.decode pkg.Cudf.package) in
       let prop =
+        (* store cudf property "version" in vertex property "cudfversion"
+         * store cudf property "number" in vertex property "version"
+         * for all other vertex properties, take the cudf property directly *)
         List.filter_map (fun (key,_,_) ->
-          if key = "version" then None else begin
-            let k = if key = "cudfversion" then "version" else key in
+            let k =
+              if key = "cudfversion" then "version"
+              else if key = "version" then "number"
+              else key
+            in
             try let value = Cudf.lookup_package_property pkg k in
               Some(key,value)
             with Not_found -> None
-          end
         ) vertex_properties
       in
-      let prop = kind :: name :: version :: binaries :: prop in
+      let prop = kind :: binaries :: cudfname :: prop in
       (* only set those attributes which are not empty *)
       List.filter_map (fun (k,v) ->
         if v = "" then None else Some(k,v)
@@ -461,7 +539,7 @@ module Graphml (U : sig val univ : Cudf.universe end)  = struct
           let sl =
             List.map (fun pid ->
               let pkg = CudfAdd.inttovar U.univ pid in
-              CudfAdd.string_of_package pkg
+              BootstrapCommon.string_of_package pkg
             ) (IntSet.elements s)
           in
           ("binaries", String.concat "," sl)
