@@ -36,26 +36,26 @@ module Make (U : sig val univ : Cudf.universe end) = struct
     | BuildGraph.SrcPkg id | BuildGraph.InstSet (id,_) ->
       CudfAdd.inttovar U.univ id
 
-  let string_of_vertex v =
+  let string_of_vertex ?(noversion=false) v =
     let pkg = pkg_of_vertex v in
-    (CudfAdd.decode pkg.Cudf.package)
+    BootstrapCommon.string_of_package ~noversion pkg
 
   let variant_of_vertex v =
     let v = BuildGraph.Unique.value v in
     match v with
     | BuildGraph.SrcPkg id -> begin
         let pkg = CudfAdd.inttovar U.univ id in
-        (`SrcPkg ((CudfAdd.decode pkg.Cudf.package), (CudfAdd.string_of_version pkg)))
+        (`SrcPkg (BootstrapCommon.debsrctuple_of_cudfpkg pkg))
       end
     | BuildGraph.InstSet (id,iss) -> begin
         let pkg = CudfAdd.inttovar U.univ id in
         let sl =
           List.map (fun pid ->
               let pkg = CudfAdd.inttovar U.univ pid in
-              ((CudfAdd.decode pkg.Cudf.package), (CudfAdd.string_of_version pkg))
+              BootstrapCommon.debbintriplet_of_cudfpkg pkg
             ) (IntSet.elements iss)
         in
-        (`InstSet (((CudfAdd.decode pkg.Cudf.package), (CudfAdd.string_of_version pkg)), sl))
+        (`InstSet (BootstrapCommon.debbintriplet_of_cudfpkg pkg, sl))
       end
 
   let intset_of_vset s =
@@ -64,8 +64,13 @@ module Make (U : sig val univ : Cudf.universe end) = struct
   let intset_of_vlist l =
     List.fold_left (fun acc v -> IntSet.add (int_of_vertex v) acc) IntSet.empty l
 
+  (* make the list unique because pkg_of_vertex omits the installation set and
+   * might thus otherwise return duplicate entries *)
   let pkglist_of_vlist l =
-    List.fold_right (fun v acc -> (pkg_of_vertex v)::acc) l []
+    BootstrapCommon.CudfSet.elements
+      (List.fold_right
+         (fun v -> BootstrapCommon.CudfSet.add (pkg_of_vertex v)) l
+         BootstrapCommon.CudfSet.empty)
 
   let variantlist_of_vlist l =
     List.fold_right (fun v acc -> (variant_of_vertex v)::acc) l []
@@ -139,7 +144,19 @@ module Make (U : sig val univ : Cudf.universe end) = struct
         | [] -> acc (* it might be that no cycles of this length can be broken *)
         | hd::tl -> begin
             let max_edge,cids = List.fold_left (fun (k1,v1) (k2,v2) ->
-               if (IntSet.cardinal v1) > (IntSet.cardinal v2) then k1,v1 else k2,v2
+                let diff = (IntSet.cardinal v1) - (IntSet.cardinal v2) in
+                if diff < 0 then k2,v2
+                else if diff > 0 then k1,v1
+                else match k1,k2 with
+                  | (sv1,{BuildGraph.depend = BuildGraph.BuildDep},iv1),(sv2,{BuildGraph.depend = BuildGraph.BuildDep},iv2) ->
+                    let diff = (BuildGraph.Unique.uid sv1) - (BuildGraph.Unique.uid sv2) in
+                    if diff < 0 then k2,v2
+                    else if diff > 0 then k1,v1
+                    else
+                      let diff = (BuildGraph.Unique.uid iv1) - (BuildGraph.Unique.uid iv2) in
+                      if diff < 0 then k2,v2
+                      else k1,v1
+                  | _ -> failwith "impossible"
             ) hd tl in
             (* end if the edge with the most cycles has zero cycles *)
             if (IntSet.cardinal cids) = 0 then
