@@ -14,7 +14,9 @@ open ExtLib
 open Common
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 let str_list_option ?(default=Some []) ?(metavar = "STRLST") =
   let sep = "," in
@@ -34,21 +36,21 @@ module Options = struct
   include BootstrapCommon.MakeOptions(struct let options = options end)
 
   let cycle_length_fas = StdOpt.int_option ~default:8 ()
-  let noindep = StdOpt.store_false ()
+  let noindep = StdOpt.store_true ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
 
   add options ~group:prog_group ~long_name:"max-length-fas" ~help:"maximum length of found cycles for fas search (default=4)" cycle_length_fas;
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
+  add options ~group:prog_group ~long_name:"drop-b-d-indep" ~help:"Drop Build-Depends-Indep dependencies" noindep;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options ;;
 
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options ;;
 
 end
@@ -82,7 +84,7 @@ let main () =
   let module BG = BuildGraph.G in
 
   let ic = open_in bgf in
-  let bg = BuildGraph.from_ic universe ic in
+  let bg = BuildGraph.from_ic universe buildarch ic in
   close_in ic;
 
   let module BGE = BuildGraphExtras.Make(struct let univ = universe end) in
