@@ -16,7 +16,9 @@ open Common
 open Debian
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 let str_list_option ?(default=Some []) ?(metavar = "STRLST") =
   let sep = "," in
@@ -39,22 +41,24 @@ module Options = struct
   include BootstrapCommon.MakeOptions(struct let options = options end)
 
   let available = StdOpt.str_option ()
-  let noindep = StdOpt.store_false ()
+  let noindep = StdOpt.store_true ()
   let custom_is_files = str_list_option ()
   let allowsrcmismatch = StdOpt.store_true ()
   let progress = StdOpt.store_true ()
   let timers = StdOpt.store_true ()
   let strongtype = StdOpt.store_true ()
   let closuretype = StdOpt.store_true ()
+  let optgraph = StdOpt.store_true ()
+  let num_cores = StdOpt.int_option ~default:1 ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
 
   add options ~group:prog_group ~short_name:'A' ~long_name:"available"
     ~help:"List of available packages (arch:all, crossed...) in control file format" available;
   add options ~group:prog_group ~long_name:"custom-is" ~help:"list of files with dependencies that should not be part of the IS" custom_is_files;
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
+  add options ~group:prog_group ~long_name:"drop-b-d-indep" ~help:"Drop Build-Depends-Indep dependencies" noindep;
   add options ~group:prog_group ~long_name:"allowsrcmismatch" ~help:("If a binary package is "^
     "without a source package but there is a source package of same name but "^ 
     "different version, match this binary package to that source package.") allowsrcmismatch;
@@ -62,13 +66,15 @@ module Options = struct
   add options ~group:prog_group ~long_name:"timers" ~help:"print timing information" timers;
   add options ~group:prog_group ~long_name:"strongtype" ~help:"generate a strong build graph (only strong dependencies)" strongtype;
   add options ~group:prog_group ~long_name:"closuretype" ~help:"generate a dependency closure build graph (all dependency relationships)" closuretype;
+  add options ~group:prog_group ~long_name:"optgraph" ~help:"generate installation sets with a minimum number of unavailable binary packages" optgraph;
+  add options ~group:prog_group ~long_name:"num_cores" ~help:"How many worker processes to run in parallel. Default: 1" num_cores;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options ;;
 
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options ;;
 end
 
@@ -78,7 +84,7 @@ let main () =
   Util.Debug.disable "Depsolver_int";
   StdDebug.all_quiet (OptParse.Opt.get Options.quiet);
   StdDebug.enable_timers (OptParse.Opt.get Options.timers) ["build_graph"];
-  StdDebug.enable_bars (OptParse.Opt.get Options.progress) ["build_graph"];
+  StdDebug.enable_bars (OptParse.Opt.get Options.progress) ["build_graph"; "Strongdeps_int.main"];
 
   let options = Options.set_deb_options () in
   let hostarch = match options.Debian.Debcudf.host with None -> "" | Some s -> s in
@@ -89,6 +95,9 @@ let main () =
   let allowsrcmismatch = OptParse.Opt.get Options.allowsrcmismatch in
   let strongtype = OptParse.Opt.get Options.strongtype in
   let closuretype = OptParse.Opt.get Options.closuretype in
+  let optgraph = OptParse.Opt.get Options.optgraph in
+  let global_constraints = not(OptParse.Opt.get Options.deb_ignore_essential) in
+  let num_cores = OptParse.Opt.get Options.num_cores in
 
   if strongtype && closuretype then
     fatal "cannot have --strongtype and --closuretype at the same time";
@@ -96,6 +105,8 @@ let main () =
   if strongtype || closuretype then begin
     if List.length custom_is_files > 0 then
       fatal "cannot have --custom-is with --strongtype or --closuretype";
+    if optgraph then
+      fatal "cannot have --optgraph with --strongtype or --closuretype";
   end;
 
   let (binlist, (fgsrclist,bgsrclist), _) = BootstrapCommon.parse_packages ~noindep Options.parse_cmdline buildarch hostarch foreignarchs posargs in
@@ -125,12 +136,12 @@ let main () =
 
   let bg =
     if strongtype then begin
-      BuildGraph.strong_graph ~available ~allowmismatch:allowsrcmismatch universe fgsl
+      BuildGraph.strong_graph ~global_constraints ~available ~allowmismatch:allowsrcmismatch ~num_cores universe fgsl
     end else if closuretype then begin
-      BuildGraph.closure_graph ~available ~allowmismatch:allowsrcmismatch universe fgsl
+      BuildGraph.closure_graph ~global_constraints ~available ~allowmismatch:allowsrcmismatch ~num_cores universe fgsl
     end else begin
       (* create a buildgraph and a sourcegraph *)
-      BuildGraph.dist_graph ~available ~allowmismatch:allowsrcmismatch custom_is_ht universe fgsl
+      BuildGraph.dist_graph ~global_constraints ~available ~allowmismatch:allowsrcmismatch ~opt:optgraph ~num_cores custom_is_ht universe fgsl
     end
   in
 
