@@ -14,7 +14,9 @@ open ExtLib
 open Common
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 let str_list_option ?(default=Some []) ?(metavar = "STRLST") =
   let sep = "," in
@@ -32,26 +34,26 @@ module Options = struct
   let options = OptParser.make ~description ~usage
   include BootstrapCommon.MakeOptions(struct let options = options end)
 
-  let noindep = StdOpt.store_false ()
+  let noindep = StdOpt.store_true ()
   let remove_weak = StdOpt.str_option ()
   let remove_reduced = str_list_option ()
   let cycle_length_fvs = StdOpt.int_option ~default:8 ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
+  add options ~group:prog_group ~long_name:"drop-b-d-indep" ~help:"Drop Build-Depends-Indep dependencies" noindep;
   add options ~group:prog_group ~long_name:"max-length-fvs"
     ~help:"maximum length of found cycles for fvs search (default=4)" cycle_length_fvs;
   add options ~group:prog_group ~long_name:"remove-weak" ~help:"path to list of weak build dependencies" remove_weak;
   add options ~group:prog_group ~long_name:"remove-reduced" ~help:"remove droppable build dependencies supplied by comma separated list of reduced dep files" remove_reduced;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options ;;
 
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options ;;
 end
 
@@ -84,7 +86,7 @@ let main () =
   let universe = Cudf.load_universe (BootstrapCommon.unique [pkglist;fgsl;bgsl]) in
 
   let ic = open_in bgf in
-  let bg = BuildGraph.from_ic universe ic in
+  let bg = BuildGraph.from_ic universe buildarch ic in
   close_in ic;
 
   let module BGE = BuildGraphExtras.Make(struct let univ = universe end) in
@@ -112,7 +114,7 @@ let main () =
   let fvs = BGE.calculate_fvs ~maxlength:maxlengthfvs reduced_deps_ht bg in
 
   BuildGraph.VertexSet.iter (fun v ->
-    Printf.printf "%s\n" (BGE.string_of_vertex v)
+    Printf.printf "%s\n" (BGE.string_of_vertex ~noversion:true v)
   ) fvs;
 ;;
 
