@@ -17,7 +17,9 @@ open ExtLib
 open Common
 open Algo
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 let timer_johnson = Util.Timer.create "GraphUtils.FindCycles.johnson"
 
