@@ -16,6 +16,7 @@ type value =
   | Int of int
   | Float of float
   | String of string
+  | Bool of bool
   | List of value_list
 and value_list = (string * value) list
 
@@ -148,6 +149,7 @@ struct
                 | ("int",n) -> Some(n, Int (int_of_string d))
                 | ("long",n) -> Some(n, Int (int_of_string d))
                 | ("float",n) -> Some(n, Float (float_of_string d))
+                | ("bool",n) -> Some(n, Bool (bool_of_string d))
                 | (t,n) -> invalid_arg (Printf.sprintf "Unsupported node type %s for attribute %s" t n)
                end with Not_found -> invalid_arg (Printf.sprintf "Cannot find node type %s" k)
              end
@@ -175,6 +177,7 @@ struct
                 | ("int",n) -> Some(n, Int (int_of_string d))
                 | ("long",n) -> Some(n, Int (int_of_string d))
                 | ("float",n) -> Some(n, Float (float_of_string d))
+                | ("bool",n) -> Some(n, Bool (bool_of_string d))
                 | (t,n) -> invalid_arg (Printf.sprintf "Unsupported edge type %s for attribute %s" t n)
                end with Not_found -> invalid_arg (Printf.sprintf "Cannot find edge type %s" k)
              end
