@@ -17,7 +17,9 @@ open Debian
 open Algo
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 module Options = struct
   open OptParse
@@ -29,15 +31,15 @@ module Options = struct
   include BootstrapCommon.MakeOptions(struct let options = options end)
 
   let addarchall = StdOpt.store_true ()
-  let noindep = StdOpt.store_false ()
+  let noindep = StdOpt.store_true ()
   let allowsrcmismatch = StdOpt.store_true ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
 
   add options ~group:prog_group ~long_name:"all" ~help:"also add source packages for Architecture:all packages" addarchall;
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
+  add options ~group:prog_group ~long_name:"drop-b-d-indep" ~help:"Drop Build-Depends-Indep dependencies" noindep;
   add options ~group:prog_group
     ~long_name:"allowsrcmismatch"
     ~help:("If a binary package is "^
@@ -45,11 +47,15 @@ module Options = struct
            "different version, match this binary package to that source package.") allowsrcmismatch;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly";"latest";"fg";"bg"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly";"latest";"fg";"bg";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options;;
 
+  include StdOptions.OutputOptions;;
+  let default = List.filter (fun e -> not (List.mem e ["outdir"; "dot"])) StdOptions.OutputOptions.default_options in
+  StdOptions.OutputOptions.add_options ~default options;;
+
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options;;
 end
 
@@ -83,11 +89,7 @@ let main () =
    * Packages.package format822 stanzas *)
   let cudftobin_table = Hashtbl.create 30000 in
   List.iter2 (fun cudfpkg -> fun binpkg ->
-      let arch =
-        try Some (Cudf.lookup_package_property cudfpkg "architecture")
-        with Not_found -> None
-      in
-      let id = (cudfpkg.Cudf.package, cudfpkg.Cudf.version, arch) in
+      let id = (cudfpkg.Cudf.package, cudfpkg.Cudf.version) in
       Hashtbl.add cudftobin_table id binpkg
     ) bl binlist;
 
@@ -104,6 +106,7 @@ let main () =
                     Cudf.provides = [(srcpkg.Cudf.package, Some (`Eq, srcpkg.Cudf.version))] }
     ) sl in
 
+  let essential = [] in
   let newbl = List.map (fun binpkg ->
       if (not addarchall) && (BootstrapCommon.pkg_is_arch_all binpkg) then
         (* do not connect arch:all packages to source packages *)
@@ -112,7 +115,8 @@ let main () =
         (* get the source package for the non-arch:all binary package *)
         let srcpkg = try BootstrapCommon.get_src_package ~allowmismatch:allowsrcmismatch universe binpkg
           with Sources.NotfoundSrc ->
-            failwith (Printf.sprintf "cannot find source for binary package %s" (CudfAdd.string_of_package binpkg))
+            failwith (Printf.sprintf "cannot find source for binary package %s"
+                        (BootstrapCommon.string_of_package binpkg))
         in
         (* connect to source package as "builds-from" *)
         let srcdep = (srcpkg.Cudf.package,Some(`Eq,srcpkg.Cudf.version)) in
@@ -127,11 +131,15 @@ let main () =
   let preamble = Debcudf.preamble in
   let preamble = CudfAdd.add_properties preamble [("issource",(`Int (Some 0)))] in
   (* any source package that builds part of the minimal builds system will draw in everything else *)
-  let request_pkgname = CudfAdd.encode "src:build-essential" in
+  let essential = List.map
+      (fun pkg -> (pkg.Cudf.package, Some (`Eq, pkg.Cudf.version)))
+      (CudfAdd.latest (Cudf.get_packages ?filter:(Some (fun pkg ->
+           pkg.Cudf.keep = `Keep_package )) universe))
+  in
   let request = { Cudf.default_request
                   with Cudf.request_id = "";
-                       Cudf.install = [(request_pkgname, None)] } in
-  let criteria = "-sum(solution,isource)" in
+                       Cudf.install = essential } in
+  let criteria = "-sum(solution,issource)" in
   let cmd = "aspcud $in $out $pref" in
   let r = Depsolver.check_request ~cmd ~criteria (preamble,newuniverse,request) in
 
@@ -152,11 +160,7 @@ let main () =
             with Not_found -> false
           in
           if not is_src then begin
-            let arch =
-              try Some (Cudf.lookup_package_property pkg "architecture")
-              with Not_found -> None
-            in
-            let id = (pkg.Cudf.package, pkg.Cudf.version, arch) in
+            let id = (pkg.Cudf.package, pkg.Cudf.version) in
             let b = Hashtbl.find cudftobin_table id in
             Debian.Printer.pp_package oc b;
             output_char oc '\n';
