@@ -14,7 +14,9 @@ open ExtLib
 open Common
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 module Options = struct
   open OptParse
@@ -25,19 +27,19 @@ module Options = struct
   let options = OptParser.make ~description ~usage
   include BootstrapCommon.MakeOptions(struct let options = options end)
 
-  let noindep = StdOpt.store_false ()
+  let noindep = StdOpt.store_true ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
+  add options ~group:prog_group ~long_name:"drop-b-d-indep" ~help:"Drop Build-Depends-Indep dependencies" noindep;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options ;;
 
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options ;;
 end
 
@@ -67,7 +69,7 @@ let main () =
   let universe = Cudf.load_universe (BootstrapCommon.unique [pkglist;fgsl;bgsl]) in
 
   let ic = open_in sgf in
-  let sg = SrcGraph.from_ic universe ic in
+  let sg = SrcGraph.from_ic universe buildarch ic in
   close_in ic;
 
   if SrcGraph.Dfs.has_cycle sg then fatal "graph is not a DAG";
@@ -80,8 +82,11 @@ let main () =
     let batch = List.sort ~cmp:(fun v1 v2 ->
       CudfAdd.compare (SGE.package_of_vertex v1) (SGE.package_of_vertex v2)
     ) batch in
-    let l = List.map SGE.string_of_vertex batch in
-    Printf.printf "%d %s\n" (i+1) (String.concat " " l)
+    let l = List.map
+        (fun v -> BootstrapCommon.string_of_package (SGE.package_of_vertex v))
+        batch
+    in
+    Printf.printf "%d %s\n" (i+1) (String.concat "," l)
   ) order;
 ;;
 
