@@ -17,7 +17,9 @@ open Debian
 open Datatypes_t
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 let str_list_option ?(default=Some []) ?(metavar = "STRLST") =
   let sep = "," in
@@ -43,9 +45,9 @@ module Options = struct
   let sapsb = StdOpt.store_true ()
   let allowsrcmismatch = StdOpt.store_true ()
   let available = StdOpt.str_option ()
-  let noindep = StdOpt.store_false ()
+  let noindep = StdOpt.store_true ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
 
@@ -56,18 +58,18 @@ module Options = struct
   add options ~group:prog_group ~long_name:"remove-reduced" ~help:"remove droppable build dependencies supplied by comma separated list of reduced dep files" remove_reduced;
   add options ~group:prog_group ~short_name:'A' ~long_name:"available"
     ~help:"List of available packages (arch:all, crossed...) in control file format" available;
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
+  add options ~group:prog_group ~long_name:"drop-b-d-indep" ~help:"Drop Build-Depends-Indep dependencies" noindep;
 
   add options ~group:prog_group ~long_name:"allowsrcmismatch" ~help:("If a binary package is "^
     "without a source package but there is a source package of same name but "^ 
     "different version, match this binary package to that source package.") allowsrcmismatch;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly"; "latest";"outfile";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options ;;
 
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options ;;
 
 end
@@ -105,11 +107,11 @@ let main () =
   let module SG = SrcGraph.G in
 
   let ic = open_in bgf in
-  let bg = BuildGraph.from_ic universe ic in
+  let bg = BuildGraph.from_ic universe buildarch ic in
   close_in ic;
 
   let ic = open_in sgf in
-  let sg = SrcGraph.from_ic universe ic in
+  let sg = SrcGraph.from_ic universe buildarch ic in
   close_in ic;
 
   (* read package list for available packages *)
@@ -140,16 +142,19 @@ let main () =
 
   let scc = List.filter_map (function [] | [_] -> None | s -> Some (BuildGraph.Oper.subgraph bg s)) (BuildGraph.Comp.scc_list bg) in
 
-  let tnvt pkg = ((CudfAdd.decode pkg.Cudf.package), (CudfAdd.string_of_version pkg)) in
-  let tnvtl = List.map tnvt in
+  (* cudf to debian binary/source converter shorthands *)
+  let c2b = BootstrapCommon.debbintriplet_of_cudfpkg in
+  let c2s = BootstrapCommon.debsrctuple_of_cudfpkg in
+  let c2bl = List.map c2b in
+  let c2sl = List.map c2s in
 
   let result = {
     srcgraph = {
       snr_vertex = SG.nb_vertex sg;
       snr_edges = SG.nb_edges sg;
-      type1cycles = List.map (fun (pkg, d1, d2) -> (tnvt pkg, tnvtl d1, tnvtl d2)) type1;
-      type2cycles = List.map (fun (pkg, d1, d2) -> (tnvt pkg, tnvtl d1, tnvtl d2)) type2;
-      type3cycles = List.map (fun (pkg, deps) -> (tnvt pkg, tnvtl deps)) type3;
+      type1cycles = List.map (fun (pkg, d1, d2) -> (c2s pkg, c2bl d1, c2bl d2)) type1;
+      type2cycles = List.map (fun (pkg, d1, d2) -> (c2s pkg, c2bl d1, c2bl d2)) type2;
+      type3cycles = List.map (fun (pkg, deps) -> (c2s pkg, c2bl deps)) type3;
     };
     buildgraph = {
       bnr_vertex = BG.nb_vertex bg;
@@ -176,12 +181,12 @@ let main () =
           sg_edges = BG.nb_edges sg;
           cycles = List.map BGE.variantlist_of_vlist cycles;
           cycleedge = List.map (fun ((v1,_,v2),c) -> ((BGE.variant_of_vertex v1), (BGE.variant_of_vertex v2)), c) cycleedge;
-          builddeps = List.map (fun (p,deps) -> tnvt p, tnvtl deps) (BGS.min_builddep sg);
-          ratio_source = List.map (fun (s,b,n,o) -> tnvt s, b, tnvtl n, tnvtl o) (BGS.ratio_source sg);
-          ratio_binary = List.map (fun (b,s,o) -> tnvt b, s, tnvtl o) (BGS.ratio_binary sg);
-          weak = List.map (fun (pkg, deps) -> tnvt pkg, tnvtl deps) (BGS.only_weak_missing weak_deps_set sg);
-          srcbinstats = List.map (fun (p,(a,b)) -> tnvt p, a, b) (BGS.get_src_bin_stats sg);
-          fas = List.map (fun (p,deps) -> tnvt (BGE.pkg_of_vertex p), tnvtl (BGE.pkglist_of_vlist deps)) fas;
+          builddeps = List.map (fun (p,deps) -> c2s p, c2bl deps) (BGS.min_builddep sg);
+          ratio_source = List.map (fun (s,b,n,o) -> c2s s, b, c2bl n, c2sl o) (BGS.ratio_source sg);
+          ratio_binary = List.map (fun (b,s,o) -> c2b b, s, c2sl o) (BGS.ratio_binary sg);
+          weak = List.map (fun (pkg, deps) -> c2s pkg, c2bl deps) (BGS.only_weak_missing weak_deps_set sg);
+          srcbinstats = List.map (fun (v,s,p) -> (BGE.variant_of_vertex v), s, p) (BGS.get_src_bin_stats sg);
+          fas = List.map (fun (p,deps) -> c2s (BGE.pkg_of_vertex p), c2bl (BGE.pkglist_of_vlist deps)) fas;
           sap = List.map (fun (p,i) -> (BGE.variant_of_vertex p), i) sap;
           sb = List.map (fun ((v1,_,v2),i) -> (BGE.variant_of_vertex v1, BGE.variant_of_vertex v2), i) sb;
         }
@@ -189,8 +194,7 @@ let main () =
     };
   } in
   
-  (*print_endline (Yojson.Safe.prettify (Datatypes_j.string_of_stats result));*)
-  print_endline (Datatypes_j.string_of_stats result);
+  print_endline (Yojson.Safe.prettify (Datatypes_j.string_of_stats result));
 ;;
 
 main ();;
