@@ -16,7 +16,9 @@ open Common
 open Debian
 open DoseparseNoRpm
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 module Options = struct
   open OptParse
@@ -28,17 +30,15 @@ module Options = struct
   let options = OptParser.make ~description ~usage
   include BootstrapCommon.MakeOptions(struct let options = options end)
 
-  let noindep = StdOpt.store_false ()
   let allowsrcmismatch = StdOpt.store_true ()
   let ignoresrclessbin = StdOpt.store_true ()
   let available = StdOpt.str_option ()
 
-  open OptParser
+  open OptParser ;;
 
   let prog_group = add_group options "Program specific options" in
 
   add options ~group:prog_group ~short_name:'A' ~long_name:"available" ~help:"List of available packages (arch:all, crossed...) in control file format" available;
-  add options ~group:prog_group ~long_name:"keep-indep" ~help:"Do not drop Build-Depends-Indep dependencies" noindep;
   add options ~group:prog_group ~long_name:"allowsrcmismatch" ~help:("If a binary package is "^
     "without a source package but there is a source package of same name but "^ 
     "different version, match this binary package to that source package.") allowsrcmismatch;
@@ -47,11 +47,15 @@ module Options = struct
     "is applied after it.") ignoresrclessbin;
 
   include StdOptions.InputOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["checkonly";"latest"])) StdOptions.InputOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["checkonly";"latest";"inputtype"])) StdOptions.InputOptions.default_options in
   StdOptions.InputOptions.add_options ~default options;;
 
+  include StdOptions.OutputOptions;;
+  let default = List.filter (fun e -> not (List.mem e ["outdir"; "dot"])) StdOptions.OutputOptions.default_options in
+  StdOptions.OutputOptions.add_options ~default options;;
+
   include StdOptions.DistribOptions;;
-  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "inputtype"])) StdOptions.DistribOptions.default_options in
+  let default = List.filter (fun e -> not (List.mem e ["deb-ignore-essential"; "deb-builds-from"])) StdOptions.DistribOptions.default_options in
   StdOptions.DistribOptions.add_options ~default options;;
 end
 
@@ -65,11 +69,10 @@ let main () =
   let buildarch = Option.get options.Debian.Debcudf.native in
   let hostarch = match options.Debian.Debcudf.host with None -> "" | Some s -> s in
   let foreignarchs = options.Debian.Debcudf.foreign in
-  let noindep = OptParse.Opt.get Options.noindep in
   let allowsrcmismatch = OptParse.Opt.get Options.allowsrcmismatch in
   let ignoresrclessbin = OptParse.Opt.get Options.ignoresrclessbin in
 
-  let binlist, (fgsrclist,bgsrclist), _ = BootstrapCommon.parse_packages ~noindep Options.parse_cmdline buildarch hostarch foreignarchs posargs in
+  let binlist, (fgsrclist,bgsrclist), _ = BootstrapCommon.parse_packages Options.parse_cmdline buildarch hostarch foreignarchs posargs in
 
   let tables = Debian.Debcudf.init_tables (fgsrclist@bgsrclist@binlist) in
   let fgsl = List.map (Debian.Debcudf.tocudf ~options tables) fgsrclist in
@@ -80,11 +83,7 @@ let main () =
    * Packages.package format822 stanzas *)
   let cudftobin_table = Hashtbl.create 30000 in
   List.iter2 (fun cudfpkg -> fun binpkg ->
-    let arch =
-      try Some (Cudf.lookup_package_property cudfpkg "architecture")
-      with Not_found -> None
-    in
-    let id = (cudfpkg.Cudf.package, cudfpkg.Cudf.version, arch) in
+    let id = (cudfpkg.Cudf.package, cudfpkg.Cudf.version) in
     Hashtbl.add cudftobin_table id binpkg
   ) bl binlist;
 
@@ -111,15 +110,11 @@ let main () =
   (* for each binary package, get the associated format822 stanza and print it
    * to stdout *)
   List.iter (fun p ->
-    let arch =
-      try Some (Cudf.lookup_package_property p "architecture")
-      with Not_found -> None
-    in
-    let id = (p.Cudf.package, p.Cudf.version, arch) in
+    let id = (p.Cudf.package, p.Cudf.version) in
     let b = Hashtbl.find cudftobin_table id in
     Debian.Printer.pp_package oc b;
     output_char oc '\n';
-  ) (BootstrapCommon.pkg_sort (CudfAdd.Cudf_set.elements bin));
+  ) (BootstrapCommon.debcudf_sort (CudfAdd.Cudf_set.elements bin));
 
   close_out oc;
 ;;
