@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
 # Copyright 2014 Helmut Grohne <helmut@subdivi.de>
 #
@@ -213,4 +213,7 @@ def main(packages_f, sources_f):
 
 if __name__ == "__main__":
     import sys
+    if len(sys.argv) != 3:
+        print("usage: %s Packages Sources" % sys.argv[0], file=sys.stderr)
+        exit(1)
     main(sys.argv[1], sys.argv[2])
