@@ -3,14 +3,22 @@
 PROGNAME=`basename $0`
 CUSTOMTMP="yes"
 OUTDIR="./"
+NUM_CORES="1"
 
 usage() {
 	echo "usage: $PROGNAME [OPTIONS] [buildarch hostarch buildpackages sources]" >&2
 	echo >&2
 	echo " -h, --help       Show this help message and exit" >&2
+	echo " -k, --keep       Keep the temporary files" >&2
 	echo " -w, --online     Produce stat.html for online consumption" >&2
 	echo " -o, --output=DIR Output directory. Default is the current directory" >&2
 	echo " -t, --tmp=DIR    Temporary directory. Default is created by mktemp(1)" >&2
+	echo " -G, --optgraph   Calculate a dependency graph where each installation" >&2
+	echo "                  set contains the minimal number of unavailable" >&2
+	echo "                  binary packages" >&2
+	echo " -j, --jobs=NUM   Number of threads or processes to run in parallel" >&2
+	echo "                  where applicable" >&2
+	echo " -I, --drop-b-d-indep Drop Build-Depends-Indep dependencies" >&2
 	echo " -v, --verbose    Be more verbose" >&2
 	echo " -d, --debug      Maximum verbosity" >&2
 	echo " -D, --develop    Execute tools from the source checkout instead of \$PATH" >&2
@@ -21,16 +29,17 @@ die() {
         if [ -n "$KEEP" ] || [ "$CUSTOMTMP" = "yes" ]; then
                 echo " + temporary files stored in $tmp" >&2
         else
-                rm -rf "$tmp"
+                rm -f "$tmp"/*
+                rm -fd $tmp
         fi
         exit 1
 }
 
 run() {
         if [ -n "$TIMERS" ]; then
-                time --format "%e seconds" $@
+                time --format "%e seconds" "$@"
         else
-                $@
+                "$@"
         fi
 }
 
@@ -40,10 +49,10 @@ trap "die \"got signal\"" 1 2 15
 getopt -T > /dev/null && exitcode=0 || exitcode=$?
 if [ $exitcode -eq 4 ]; then
 	# GNU enhanced getopt is available
-	ARGS=`getopt --long help,online,output:,tmp:,verbose,debug,develop --options hwo:t:vdD -- "$@"`
+	ARGS=`getopt --long help,online,output:,tmp:,optgraph,jobs:,drop-b-d-indep,verbose,debug,develop --options hwo:t:Gj:IvdD -- "$@"`
 else
 	# Original getopt is available
-	ARGS=`getopt hwo:t:vdD "$@"`
+	ARGS=`getopt hwo:t:Gj:IvdD "$@"`
 fi
 
 if [ $? -ne 0 ]; then
@@ -55,9 +64,13 @@ eval set -- $ARGS
 while [ $# -gt 0 ]; do
 	case "$1" in
 		-h | --help)    usage; exit 0;;
+		-k | --keep)    KEEP=true;;
 		-w | --online)  ONLINE="--online";;
 		-o | --output)  OUTDIR="$2"; shift;;
 		-t | --tmp)     tmp="$2"; shift;;
+		-G | --optgraph) OPTGRAPH=--optgraph;;
+		-j | --jobs)    NUM_CORES="$2"; shift;;
+		-I | --drop-b-d-indep) DROPBDINDEP=--drop-b-d-indep;;
 		-v | --verbose) VERBOSE=--verbose;;
 		-d | --debug)   set -x;;
 		-D | --develop) DEVELOP=true;;
@@ -71,7 +84,6 @@ bin_buildcheck=dose-builddebcheck
 if [ -n "$DEVELOP" ]; then
 	droppable=./droppable
 	[ -f ./bin2src.native ] && bin_bin2src=./bin2src.native || bin_bin2src=./bin2src.d.byte
-	[ -f ./build-closure.native ] && bin_build_closure=./build-closure.native || bin_build_closure=./build-closure.d.byte
 	[ -f ./build-fixpoint.native ] && bin_build_fixpoint=./build-fixpoint.native || bin_build_fixpoint=./build-fixpoint.d.byte
 	[ -f ./buildgraph2srcgraph.native ] && bin_buildgraph_to_srcgraph=./buildgraph2srcgraph.native || bin_buildgraph_to_srcgraph=./buildgraph2srcgraph.d.byte
 	[ -f ./clean-repository.native ] && bin_clean_repository=./clean-repository.native || bin_clean_repository=./clean-repository.d.byte
@@ -95,10 +107,10 @@ if [ -n "$DEVELOP" ]; then
 	bin_profile_build_fvs=./tools/profile-build-fvs.py
 	bin_stat_html=./tools/stat-html.py
 	bin_download_pkgsrc=./tools/download-pkgsrc.sh
+	bin_buildgraph2packages=./tools/buildgraph2packages.py
 else
 	droppable=/usr/share/botch/droppable
 	bin_bin2src=botch-bin2src
-	bin_build_closure=botch-build-closure
 	bin_build_fixpoint=botch-build-fixpoint
 	bin_buildgraph_to_srcgraph=botch-buildgraph2srcgraph
 	bin_clean_repository=botch-clean-repository
@@ -122,6 +134,7 @@ else
 	bin_profile_build_fvs=botch-profile-build-fvs
 	bin_stat_html=botch-stat-html
 	bin_download_pkgsrc=botch-download-pkgsrc
+	bin_buildgraph2packages=botch-buildgraph2packages
 fi
 
 
@@ -157,17 +170,23 @@ else
 	CUSTOMTMP="no"
 fi
 
+if [ "$KEEP" != "true" ] && [ -n "$(ls -A $tmp)" ]; then
+       echo "$tmp is not empty and you did not specify --keep" >&2
+       echo "refusing to run and delete that directory" >&2
+       exit 1
+fi
+
 echo " + running clean repository" >&2
 
 # clean up buildpackages repository
-$bin_clean_repository $VERBOSE --deb-native-arch=$BUILDARCH \
+$bin_clean_repository $VERBOSE $DROPBDINDEP --deb-native-arch=$BUILDARCH \
 	$buildpackages $sources \
 	> $tmp/buildpackages_clean
 buildpackages="$tmp/buildpackages_clean"
 
 echo " + running bin2src" >&2
 
-run $bin_bin2src $VERBOSE --deb-native-arch=$BUILDARCH $buildpackages $sources \
+run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$BUILDARCH $buildpackages $sources \
 	> $tmp/sources_clean || die "bin2src failed"
 sources="$tmp/sources_clean"
 
@@ -199,20 +218,42 @@ run $bin_coinstall $VERBOSE \
 
 echo " + running bin2src..." >&2
 
-run $bin_bin2src $VERBOSE --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
+run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
         "$tmp/minimal-$HOSTARCH" "$sources" \
         > "$tmp/crossed-src" || die "bin2src failed"
 
-echo " + running build-closure..." >&2
+echo " + running grep-dctrl..." >&2
 
-run $bin_build_closure $VERBOSE --deb-native-arch=$HOSTARCH \
-    -m "$tmp/minimal-$HOSTARCH" $ALLOWSRCMISMATCH --bg "$sources" \
-    "$hostpackages" "$tmp/crossed-src" \
-    > "$tmp/minimal-closure" || die "build-closure failed"
+run grep-dctrl -FArchitecture all "$hostpackages" \
+	> "$tmp/available-all" || die "grep-dctrl failed"
+
+echo " + creating a self-contained repository graph..." >&2
+
+# we ignore essential because it is already part of $tmp/minimal-$HOSTARCH
+# this also means that we have to create the union of the results of this with
+# $tmp/minimal-$HOSTARCH
+# the graph is started from all source packages that have been crossed before
+# and are thus creating the binary packages in $tmp/minimal-$HOSTARCH
+run $bin_create_graph $VERBOSE $DROPBDINDEP --num_cores=$NUM_CORES \
+	--deb-ignore-essential --progress --timers \
+	-A "$tmp/available-all" $ALLOWSRCMISMATCH --deb-native-arch=$HOSTARCH \
+	--bg $sources $hostpackages $tmp/crossed-src \
+	> "${OUTDIR}/selfcontained_repo.xml" || die "create graph failed"
+
+echo " + running buildgraph2packages" >&2
+
+run $bin_buildgraph2packages "${OUTDIR}/selfcontained_repo.xml" "$hostpackages" \
+	> "$tmp/minimal-closure" || die "buildgraph2packages failed"
+
+echo " + union..." >&2
+
+run $bin_packages_union $VERBOSE "$tmp/minimal-closure" \
+	"$tmp/minimal-$HOSTARCH" "$tmp/minimal-closure" \
+	|| die "union failed"
 
 echo " + running bin2src..." >&2
 
-run $bin_bin2src $VERBOSE --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
+run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
        "$tmp/minimal-closure" "$sources" \
        > "$tmp/minimal-closure-src" || die "bin2src failed"
 
@@ -305,7 +346,7 @@ run grep-dctrl -X \( -FArchitecture all --or --not -FMulti-Arch foreign \) "$hos
 hostpackages="$tmp/hostpackages-no-ma-foreign"
 
 echo " + bin2src..." >&2
-run $bin_bin2src $VERBOSE --deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH $ALLOWSRCMISMATCH \
+run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH $ALLOWSRCMISMATCH \
         "$tmp/not-available" "$sources" \
         > "$tmp/tocompile" || die "bin2src failed"
 
@@ -317,7 +358,7 @@ run $bin_packages_union $VERBOSE "$tmp/available-all" $buildpackages \
 # running fixpoint
 echo " + running build-fixpoint..." >&2
 
-run $bin_build_fixpoint $VERBOSE --deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH \
+run $bin_build_fixpoint $VERBOSE $DROPBDINDEP --deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH \
 	$ALLOWSRCMISMATCH -A "$tmp/available-all-build" \
 	--output-order="${OUTDIR}/order1.lst" "$buildpackages" \
 	"$tmp/crossbuild-essential-${HOSTARCH}" "$hostpackages" \
@@ -345,18 +386,22 @@ run $bin_packages_difference $VERBOSE "$tmp/tocompile" "$tmp/compilable-src" \
 # Packages and Sources files to create the exact same cudf universe
 UNIVERSE="--deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH --bg $sources $buildpackages "$tmp/crossbuild-essential-${HOSTARCH}" $hostpackages $tmp/tocompile"
 
-run $bin_create_graph $VERBOSE --progress --timers $STRONGARG \
+# we can use deb-ignore-essential here because the set of available packages
+# was made sure to contain a coinstallation set of essential earlier in this
+# script
+run $bin_create_graph $VERBOSE $DROPBDINDEP $OPTGRAPH --progress --timers \
+	--num_cores=$NUM_CORES --deb-ignore-essential $STRONGARG \
 	-A "$tmp/available" $ALLOWSRCMISMATCH $UNIVERSE \
 	> "${OUTDIR}/buildgraph.xml" || die "create graph failed"
 
 echo " + running buildgraph2srcgraph..." >&2
 
-run $bin_buildgraph_to_srcgraph $VERBOSE "${OUTDIR}/buildgraph.xml" $UNIVERSE \
+run $bin_buildgraph_to_srcgraph $VERBOSE $DROPBDINDEP "${OUTDIR}/buildgraph.xml" $UNIVERSE \
 	> "${OUTDIR}/srcgraph.xml" || die "buildgraph2srcgraph failed"
 
 droplist="--remove-weak=${droppable}/weak-build-dependencies.list --remove-reduced=${droppable}/pehjota.list,${droppable}/thorsten-glaser.list,${droppable}/daniel-schepler.list,${droppable}/gentoo.list,${droppable}/wookey.list,${droppable}/selfcycles.list"
 
-run $bin_print_stats $VERBOSE --max-length=4 --max-length-fas=8 $ALLOWSRCMISMATCH \
+run $bin_print_stats $VERBOSE $DROPBDINDEP --max-length=4 --max-length-fas=8 $ALLOWSRCMISMATCH \
 	$SAPSB $droplist -A "$tmp/available" "${OUTDIR}/buildgraph.xml" \
 	"${OUTDIR}/srcgraph.xml" $UNIVERSE > "${OUTDIR}/stats.json" || die "print stats failed"
 
@@ -366,11 +411,12 @@ run $bin_stat_html $VERBOSE $ONLINE "${OUTDIR}/stats.json" > "${OUTDIR}/stats.ht
 
 echo " + extracting the calculated feedback arc set..." >&2
 
-run $bin_fasofstats $VERBOSE "${OUTDIR}/stats.json" > "${OUTDIR}/remove.list" || die "fasofstats failed"
+run $bin_fasofstats $VERBOSE "${OUTDIR}/stats.json" | sort > "${OUTDIR}/remove.list" || die "fasofstats failed"
 
 echo " + extract strongly connected components..." >&2
 
-run $bin_extract_scc $VERBOSE --outdir=${OUTDIR} "${OUTDIR}/buildgraph.xml" cyclic \
+run $bin_extract_scc $VERBOSE --outdir=${OUTDIR} "${OUTDIR}/buildgraph.xml" \
+	cyclic --outfnameattr=cudfname --outfnameverts type:src \
 	| sort > "${OUTDIR}/fas" || die "extract scc failed"
 
 echo " + calculate feedback vertex sets..." >&2
@@ -379,7 +425,7 @@ alldrop="$droplist,${OUTDIR}/remove.list"
 
 echo > "${OUTDIR}/feedback_vertex_set.list"
 while read g; do
-	run $bin_find_fvs $VERBOSE $alldrop "${OUTDIR}/$g" $UNIVERSE \
+	run $bin_find_fvs $VERBOSE $DROPBDINDEP $alldrop "${OUTDIR}/$g" $UNIVERSE \
 		|| die "find fvs failed"
 done < "${OUTDIR}/fas" | sort > "${OUTDIR}/feedback_vertex_set.list"
 
@@ -392,12 +438,12 @@ run $bin_profile_build_fvs $VERBOSE $alldrop "${OUTDIR}/buildgraph.xml" \
 
 echo " + convert to srcgraph..." >&2
 
-run $bin_buildgraph_to_srcgraph $VERBOSE "${OUTDIR}/buildgraph_acyclic.xml" $UNIVERSE \
+run $bin_buildgraph_to_srcgraph $VERBOSE $DROPBDINDEP "${OUTDIR}/buildgraph_acyclic.xml" $UNIVERSE \
 	> "${OUTDIR}/srcgraph_acyclic.xml" || die "buildgraph2srcgraph failed"
 
 echo " + calculate partial order..." >&2
 
-run $bin_partial_order $VERBOSE "${OUTDIR}/srcgraph_acyclic.xml" $UNIVERSE \
+run $bin_partial_order $VERBOSE $DROPBDINDEP "${OUTDIR}/srcgraph_acyclic.xml" $UNIVERSE \
 	> "${OUTDIR}/order2.lst" || die "partial order failed"
 
 echo " + build order is saved in ${OUTDIR}/order1.lst and ${OUTDIR}/order2.lst" >&2
@@ -414,5 +460,6 @@ done < "${OUTDIR}/fas"
 if [ -n "$KEEP" ] || [ "$CUSTOMTMP" = "yes" ]; then
 	echo " + temporary files stored in $tmp" >&2
 else
-	rm -rf "$tmp"
+	rm -f "$tmp"/*
+	rm -fd $tmp
 fi
