@@ -2,17 +2,16 @@
 
 from __future__ import print_function
 from collections import defaultdict
-from datetime import datetime
 from functools import cmp_to_key
 import sys
 sys.path.append('/usr/share/botch')
-from util import write_plain, read_yaml_file, cmp, parse_dose_yaml
+from util import write_plain, read_yaml_file, cmp, parse_dose_yaml_mc
 from util import read_tag_file
 import os
 
 
 def print_package(arg, bin2src, source=None, color=True):
-    pkgname, version = arg
+    pkgname, version, vpkg = arg
     sn, _ = src_of_pkg(arg, bin2src)
     if source:
         highlight = (sn == source[0])
@@ -30,16 +29,17 @@ def print_package(arg, bin2src, source=None, color=True):
         else:
             pkgstring = 'src:%s' % pkgname
     else:
-        arch, pkgname = pkgname.split(':', 1)
+        pkgname, arch = pkgname.split(':', 1)
         bugs = binpkgbugs.get(pkgname)
         if bin2src.get((pkgname, 'all', version)):
             arch = 'all'
         if version:
-            pkgstring = '<a title="%s">%s:%s</a>' % (version, pkgname, arch)
+            pkgstring = '<a title="%s:%s (= %s)">%s</a>' % (pkgname, arch,
+                                                            version, pkgname)
         else:
             pkgstring = '%s:%s' % (pkgname, arch)
     if highlight:
-        pkgstring = '<span style="color:#090;font-weight:bold">'+pkgstring
+        pkgstring = '<span style="color:#090;font-weight:bold">' + pkgstring
         pkgstring += '</span>'
     if bugs:
         bugstring = " ".join('<a href="http://bugs.debian.org/%s">%s</a>' % (b,
@@ -54,18 +54,23 @@ def print_package(arg, bin2src, source=None, color=True):
         wnpp = wnpp_stats.get(sn)
         if wnpp:
             t, i = wnpp
-            pkgstring = '%s (%s: <a href="http://bugs.debian.org/%s">#%s</a>)' \
-                        % (pkgstring, t, i, i)
+            pkgstring = \
+                '%s (%s: <a href="http://bugs.debian.org/%s">#%s</a>)' \
+                % (pkgstring, t, i, i)
 
+    if vpkg:
+        pkgstring += '<a title="%s"> → </a>' % vpkg
     return pkgstring
 
 
-def by_srcname(a, b):
+def by_srcname_chain(a, b):
     num_src = cmp(len(b[1]), len(a[1]))
     if num_src:
         return num_src
-    else:
-        return cmp(min(a[1]), min(b[1]))
+    cmp_src = cmp(min(a[1]), min(b[1]))
+    if cmp_src:
+        return cmp_src
+    return cmp(a[0], b[0])
 
 
 def by_affected_sources_missing(a, b):
@@ -85,7 +90,7 @@ def by_affected_sources_conflict(a, b):
     if num_src:
         return num_src
     else:
-        return cmp(a[0][0], b[0][0])
+        return cmp(a[0], b[0])
 
 
 def print_header(outfile, description, btsuser, btstag):
@@ -112,21 +117,27 @@ def print_header(outfile, description, btsuser, btstag):
               "the <a href=\"https://bugs.debian.org/cgi-bin/pkgreport.cgi?t" +
               "ag=%s;users=%s\"" % (btstag, btsuser) +
               ">Debian bts</a></p>", file=outfile)
+    print("<p>Hover over a package name with your cursor for architecture and "
+          "version information. Hovering over the arrows in the depchain "
+          "columns will show the dependency that led from one package in the "
+          "chain to the next.</p>", file=outfile)
 
 
 def print_missing(outfile, missing, bin2src, source=None):
     print("<h2>missing</h2>", file=outfile)
-    print("<p>The packages in the third column cannot satisfy their " +
-          "(possibly transitive) dependencies because the " +
-          "binary package in the last column. This is mostly because the " +
-          "binary package in the last column is Multi-Arch:no. Some of " +
-          "these packages need to be Multi-Arch:foreign instead. In some " +
-          "other cases, Build-Depends can be annotated with :native. The " +
-          "depchains column shows the dependency chain from the packages in " +
-          "the third column to the binary package in the last column. The " +
-          "\"(*)\" placeholder in the depchains column represents any " +
-          "package in the third column.</p>",
-          file=outfile)
+
+    print("<p>The packages in the third column cannot satisfy their (possibly "
+          "transitive) dependencies because of the unsatisfied dependency in "
+          "the last column. This is mostly because the binary package "
+          "providing the dependency in the last column is Multi-Arch:no. Some "
+          "of these packages need to be Multi-Arch:foreign instead. In some "
+          "other cases, Build-Depends can be annotated with :native. The "
+          "depchains column shows the dependency chain(s) from the packages "
+          "in the third column to the unsatisfied dependency in the last "
+          "column. The \"(*)\" placeholder in the depchains column represents "
+          "any package in the third column. Hovering over the arrows in the "
+          "depchains column with your cursor will show the dependency that "
+          "led from one package in the chain to the next.</p>", file=outfile)
     print("<p>The output is first grouped by the shared unsatisfied " +
           "dependency (last column) and then by shared dependency chain " +
           "(fourth column). The groups are sorted by the number of " +
@@ -138,8 +149,8 @@ def print_missing(outfile, missing, bin2src, source=None):
           "ndencies</th><th>Depchains</th>" +
           "<th>Unsatisfied dependency</th></tr>", file=outfile)
 
-    for pkg, v in sorted(list(missing.items()),
-                         key=cmp_to_key(by_affected_sources_missing)):
+    for missing_vpkg, v in sorted(list(missing.items()),
+                                  key=cmp_to_key(by_affected_sources_missing)):
         rows = len(v)
         for c in list(v.keys()):
             if c:
@@ -149,14 +160,15 @@ def print_missing(outfile, missing, bin2src, source=None):
                                                            in v.values()])),
               file=outfile)
         first = True
-        for chain, v in sorted(list(v.items()), key=cmp_to_key(by_srcname)):
+        for chain, v in sorted(list(v.items()),
+                               key=cmp_to_key(by_srcname_chain)):
             if chain is None:
                 maxnumchains = 1
             else:
                 maxnumchains = len(chain)
             if chain:
-                c = " → ".join([print_package(t, bin2src, source)
-                                 for t in chain[0]])
+                c = "".join([print_package(t, bin2src, source)
+                             for t in chain[0]])
             else:
                 c = ""
             if not first:
@@ -168,15 +180,15 @@ def print_missing(outfile, missing, bin2src, source=None):
                   file=outfile)
             if first:
                 print('<td rowspan="%d">%s</td></tr>'
-                      % (rows, print_package(pkg, bin2src, source)),
+                      % (rows, missing_vpkg),
                       file=outfile)
             else:
                 print('</tr>', file=outfile)
             if chain:
                 for i in range(1, len(chain)):
                     print('<tr><td>%s</td></tr>' %
-                          (" → ".join([print_package(t, bin2src, source)
-                                        for t in chain[i]])),
+                          ("".join([print_package(t, bin2src, source)
+                                    for t in chain[i]])),
                           file=outfile)
             first = False
     print("</table>", file=outfile)
@@ -184,13 +196,15 @@ def print_missing(outfile, missing, bin2src, source=None):
 
 def print_conflict(outfile, conflict, bin2src, source=None):
     print("<h2>conflict</h2>", file=outfile)
-    print("<p>The packages in the third column cannot satisfy their " +
-          "dependencies because the two binary packages in the " +
-          "last column cannot be installed together. The two " +
-          "depchain columns show the paths from the packages in the " +
-          "third column to each of the conflicting binary packages in the " +
-          "last column, respectively. Sometimes, multiple " +
-          "dependency chains exist.<p>", file=outfile)
+    print("<p>The packages in the third column cannot satisfy their (possibly "
+          "transitive) dependencies because the last package(s) in the first "
+          "depchain have an unsatisfied conflict which is shown in the last "
+          "column. The second depchain column shows the dependency chain(s) "
+          "to the package which the last package(s) in the first depchain "
+          "conflict with. Sometimes, multiple dependency chains sharing the "
+          "same conflict exist. Hovering over the arrows in the depchains "
+          "column with your cursor will show the dependency that led from one "
+          "package in the chain to the next.</p>", file=outfile)
     print("<p>The output is first grouped by the shared conflicting " +
           "dependency (last column) and then by the shared dependency " +
           "chains (fourth and fifth column). The groups are sorted by the " +
@@ -204,9 +218,9 @@ def print_conflict(outfile, conflict, bin2src, source=None):
         "dencies</th><th>Depchain " +
         "1</th><th>Depchain2</th><th>Conflict</th></tr>", file=outfile)
 
-    for (pkg1, pkg2), v in sorted(list(conflict.items()),
-                                  key=cmp_to_key(
-                                      by_affected_sources_conflict)):
+    for conflict_vpkg, v in sorted(list(conflict.items()),
+                                   key=cmp_to_key(
+                                       by_affected_sources_conflict)):
         rows = len(v)
         for c1, c2 in list(v.keys()):
             if c1 is not None:
@@ -224,7 +238,7 @@ def print_conflict(outfile, conflict, bin2src, source=None):
               file=outfile)
         first = True
         for (chain1, chain2), v in sorted(list(v.items()),
-                                          key=cmp_to_key(by_srcname)):
+                                          key=cmp_to_key(by_srcname_chain)):
             if chain1 is None:
                 maxnumchains = len(chain2)
             elif chain2 is None:
@@ -232,13 +246,13 @@ def print_conflict(outfile, conflict, bin2src, source=None):
             else:
                 maxnumchains = max(len(chain1), len(chain2))
             if chain1:
-                c1 = " → ".join([print_package(t, bin2src, source)
-                                  for t in chain1[0]])
+                c1 = "".join([print_package(t, bin2src, source)
+                              for t in chain1[0]])
             else:
                 c1 = ""
             if chain2:
-                c2 = " → ".join([print_package(t, bin2src, source)
-                                  for t in chain2[0]])
+                c2 = "".join([print_package(t, bin2src, source)
+                              for t in chain2[0]])
             else:
                 c2 = ""
             if not first:
@@ -249,21 +263,20 @@ def print_conflict(outfile, conflict, bin2src, source=None):
                                              for t in sorted(v)]), c1, c2),
                   file=outfile)
             if first:
-                print('<td rowspan="%d">%s ↔ %s</td></tr>' %
-                      (rows, print_package(pkg1, bin2src, source),
-                       print_package(pkg2, bin2src, source)),
+                print('<td rowspan="%d">%s</td></tr>' %
+                      (rows, conflict_vpkg),
                       file=outfile)
             else:
                 print('</tr>', file=outfile)
             for i in range(1, maxnumchains):
                 if chain1 is not None and len(chain1) >= i + 1:
-                    c1 = " → ".join([print_package(t, bin2src, source)
-                                      for t in chain1[i]])
+                    c1 = "".join([print_package(t, bin2src, source)
+                                  for t in chain1[i]])
                 else:
                     c1 = ""
                 if chain2 is not None and len(chain2) >= i + 1:
-                    c2 = " → ".join([print_package(t, bin2src, source)
-                                      for t in chain2[i]])
+                    c2 = "".join([print_package(t, bin2src, source)
+                                  for t in chain2[i]])
                 else:
                     c2 = ""
                 print('<tr><td>%s</td><td>%s</td></tr>' %
@@ -272,42 +285,44 @@ def print_conflict(outfile, conflict, bin2src, source=None):
     print("</table>", file=outfile)
 
 
-def print_footer(outfile, srcs=[], srcsdir=""):
+def print_footer(outfile, timestamp, srcs=[], srcsdir=""):
     if srcs:
         print("<h2>Affected source packages:</h2>", file=outfile)
         for src in sorted([s for s in srcs if s is not None]):
             print("<a href=\"%s/%s.html\">%s</a>" % (srcsdir, src, src),
                   file=outfile)
 
+    url = \
+        "https://gitlab.mister-muffin.de/debian-bootstrap/bootstrap_debian_net"
     footer = """
     <hr />
     <p>The JSON data used to generate these pages was computed using botch, the
     bootstrap/build ordering tool chain. The source code of botch can be
     redistributed under the terms of the LGPL3+ with an OCaml linking
     exception. The source code can be retrieved from <a
-    href="https://gitorious.org/debian-bootstrap/botch">
-    https://gitorious.org/debian-bootstrap/botch</a></p>
+    href="https://gitlab.mister-muffin.de/debian-bootstrap/botch">
+    https://gitlab.mister-muffin.de/debian-bootstrap/botch</a></p>
 
     <p>The html pages were generated by code which can be retrieved from <a
-    href="https://gitorious.org/debian-bootstrap/bootstrap_debian_net">
-    https://gitorious.org/debian-bootstrap/bootstrap_debian_net</a> and which
-    can be redistributed under the terms of the AGPL3+</p>
+    href="%s"> %s</a> and which can be redistributed under the terms of the
+    AGPL3+</p>
 
     <p>For questions and bugreports please contact j [dot] schauer [at] email
     [dot] de.</p>
-    """
+    """ % (url, url)
 
-    print("<p>generated: %s</p>" % (datetime.now().isoformat()), file=outfile)
+    if timestamp:
+        print("<p>generated: %s</p>" % timestamp, file=outfile)
     print("%s</body></html>" % footer, file=outfile)
 
 
 def src_of_pkg(t, bin2src):
-    n, v = t
+    n, v, _ = t
     if n.startswith("src:"):
         return n[4:], v
     if n == "":
         return (None, None)
-    a, n = n.split(":", 1)
+    n, a = n.split(":", 1)
     res = bin2src.get((n, a, v))
     if res:
         return res
@@ -335,8 +350,9 @@ def print_top10summary(outfile, missing, conflict, bin2src, source=None):
 
         print("<table><tr><th># of packages per missing</th><th>Unsatisfied " +
               "dependency</th></tr>", file=outfile)
-        for pkg, v in sorted(list(missing.items()),
-                             key=cmp_to_key(by_affected_sources_missing))[:10]:
+        for missing_vpkg, v in sorted(list(missing.items()),
+                                      key=cmp_to_key(
+                                          by_affected_sources_missing))[:10]:
             rows = len(v)
             for c in list(v.keys()):
                 if c:
@@ -344,9 +360,7 @@ def print_top10summary(outfile, missing, conflict, bin2src, source=None):
             print('<tr><td>%d</td><td>%s</td></tr>' % (sum([len(s)
                                                             for s
                                                             in v.values()]),
-                                                       print_package(pkg,
-                                                                     bin2src,
-                                                                     source)),
+                                                       missing_vpkg),
                   file=outfile)
         print("</table>", file=outfile)
 
@@ -355,77 +369,68 @@ def print_top10summary(outfile, missing, conflict, bin2src, source=None):
         print("<table><tr><th># of packages per conflict</th>" +
               "<th>Conflict</th></tr>", file=outfile)
 
-        for (pkg1, pkg2), v in sorted(list(conflict.items()),
-                                      key=cmp_to_key(
-                                          by_affected_sources_conflict))[:10]:
-            print('<tr><td>%d</td><td>%s ↔ %s</td></tr>'
+        for conflict_vpkg, v in sorted(list(conflict.items()),
+                                       key=cmp_to_key(
+                                           by_affected_sources_conflict))[:10]:
+            print('<tr><td>%d</td><td>%s</td></tr>'
                   % (sum([len(s) for s in v.values()]),
-                      print_package(pkg1, bin2src, source),
-                      print_package(pkg2, bin2src, source)),
+                      conflict_vpkg),
                   file=outfile)
         print("</table>", file=outfile)
 
 
 def create_srcpkg_pages(missing, conflict, bin2src, srcsdir, description,
-                        btsuser, btstag):
+                        btsuser, btstag, timestamp):
     srcsdict = defaultdict(lambda: defaultdict(lambda:
                                                defaultdict(lambda:
                                                            defaultdict(dict))))
 
     # now go through all missing and conflict and copy them to the source
     # packages that associate with them
-    for pkg, v in missing.items():
-        sn, sv = src_of_pkg(pkg, bin2src)
-        srcsdict[sn][sv]['missing'][pkg] = v
+    # it is not possible to find a source package for a vpkg (especially if
+    # it's missing)
+    for vpkg, v in missing.items():
         for chain, v in v.items():
             for c in chain:
                 for p in c:
                     sn, sv = src_of_pkg(p, bin2src)
                     if not sn:
                         continue
-                    srcsdict[sn][sv]['missing'][pkg][chain] = v
+                    srcsdict[sn][sv]['missing'][vpkg][chain] = v
             for p in v:
                 sn, sv = src_of_pkg(p, bin2src)
-                srcsdict[sn][sv]['missing'][pkg][chain] = v
+                srcsdict[sn][sv]['missing'][vpkg][chain] = v
 
-    for (pkg1, pkg2), v in conflict.items():
-        sn, sv = src_of_pkg(pkg1, bin2src)
-        srcsdict[sn][sv]['conflict'][(pkg1, pkg2)] = v
-        sn, sv = src_of_pkg(pkg2, bin2src)
-        srcsdict[sn][sv]['conflict'][(pkg1, pkg2)] = v
+    for vpkg, v in conflict.items():
         for (chain1, chain2), v in v.items():
             for c in chain1:
                 for p in c:
                     sn, sv = src_of_pkg(p, bin2src)
                     if not sn:
                         continue
-                    srcsdict[sn][sv]['conflict'][(pkg1, pkg2)][(chain1,
-                                                                chain2)] = v
+                    srcsdict[sn][sv]['conflict'][vpkg][(chain1, chain2)] = v
             for c in chain2:
                 for p in c:
                     sn, sv = src_of_pkg(p, bin2src)
                     if not sn:
                         continue
-                    srcsdict[sn][sv]['conflict'][(pkg1, pkg2)][(chain1,
-                                                                chain2)] = v
+                    srcsdict[sn][sv]['conflict'][vpkg][(chain1, chain2)] = v
             for p in v:
                 sn, sv = src_of_pkg(p, bin2src)
-                srcsdict[sn][sv]['conflict'][(pkg1, pkg2)][(chain1,
-                                                            chain2)] = v
+                srcsdict[sn][sv]['conflict'][vpkg][(chain1, chain2)] = v
 
     if not os.path.isdir(srcsdir):
         os.mkdir(srcsdir)
     for srcpkg, versions in srcsdict.items():
         if not srcpkg:
             continue
-        fname = os.path.join(srcsdir, srcpkg)+".html"
+        fname = os.path.join(srcsdir, srcpkg) + ".html"
         with open(fname, "w") as srcpkgpage:
             print_header(srcpkgpage, description, btsuser, btstag)
             for ver in versions.keys():
-                print("<h1>%s</h1>" % print_package(("src:"+srcpkg, ver),
-                                                    bin2src,
-                                                    color=False),
-                      file=srcpkgpage)
+                print("<h1>%s</h1>" % print_package(
+                    ("src:" + srcpkg, ver, None), bin2src, color=False),
+                    file=srcpkgpage)
                 print_top10summary(srcpkgpage, versions[ver].get('missing'),
                                    versions[ver].get('conflict'), bin2src,
                                    (srcpkg, ver))
@@ -435,14 +440,14 @@ def create_srcpkg_pages(missing, conflict, bin2src, srcsdir, description,
                 if versions[ver].get('conflict'):
                     print_conflict(srcpkgpage, versions[ver]['conflict'],
                                    bin2src, (srcpkg, ver))
-            print_footer(srcpkgpage)
+            print_footer(srcpkgpage, timestamp)
 
     return srcsdict.keys()
 
 
 def dose2html(yamlin, outfile, description="", srcpkgbugs=set(),
               binpkgbugs=set(), btsuser=None, btstag=None, srcsdir=None,
-              packages=[], verbose=False):
+              packages=[], timestamp=None, verbose=False):
     bin2src = dict()
 
     for pkg in packages:
@@ -470,7 +475,7 @@ def dose2html(yamlin, outfile, description="", srcpkgbugs=set(),
             bin2src[(pname, arch, ver)] = (srcname, srcver)
             bin2src[(pname, arch, None)] = (srcname, None)
 
-    missing, conflict, _ = parse_dose_yaml(yamlin)
+    missing, conflict = parse_dose_yaml_mc(yamlin)
 
     print_header(outfile, description, btsuser, btstag)
     print_top10summary(outfile, missing, conflict, bin2src)
@@ -479,10 +484,10 @@ def dose2html(yamlin, outfile, description="", srcpkgbugs=set(),
 
     if srcsdir and packages:
         srcpkgs = create_srcpkg_pages(missing, conflict, bin2src, srcsdir,
-                                      description, btsuser, btstag)
-        print_footer(outfile, srcpkgs, srcsdir)
+                                      description, btsuser, btstag, timestamp)
+        print_footer(outfile, timestamp, srcpkgs, srcsdir)
     else:
-        print_footer(outfile)
+        print_footer(outfile, timestamp)
 
 if __name__ == '__main__':
     import argparse
@@ -501,9 +506,12 @@ if __name__ == '__main__':
                         'individual source package overviews')
     parser.add_argument('--wnpp', action='store_true',
                         help='retrieve and print wnpp status for packages')
-    parser.add_argument('--packages', type=read_tag_file,
+    parser.add_argument('--packages', type=read_tag_file, default=[],
                         help='Packages file to create a mapping from binary ' +
                              'to source packages')
+    parser.add_argument('--timestamp',
+                        help='put a freeform timestamp string at the bottom ' +
+                             'of the generated HTML')
     parser.add_argument(
         '-v', '--verbose', action='store_true', help='be verbose')
     args = parser.parse_args()
@@ -532,9 +540,9 @@ if __name__ == '__main__':
         #            srcpkgbugs[s].append(k)
         import subprocess
         res = subprocess.check_output(
-            "bts select users:%s " % args.btsuser
-            + "tag:%s | bts status " % args.btstag
-            + "fields:source,package,bug_num file:-", shell=True)
+            "bts select users:%s " % args.btsuser +
+            "tag:%s status:open | bts status " % args.btstag +
+            "fields:source,package,bug_num file:-", shell=True)
         for b in res.decode().split("\n\n"):
             if not b:
                 continue
@@ -569,4 +577,4 @@ if __name__ == '__main__':
     with args.htmlout as f:
         dose2html(args.yamlin, f, args.desc, srcpkgbugs, binpkgbugs,
                   args.btsuser, args.btstag, args.srcsdir, args.packages,
-                  args.verbose)
+                  args.timestamp, args.verbose)
