@@ -5,22 +5,21 @@ import sys
 sys.path.append('/usr/share/botch')
 from util import read_json_file
 
+
 # string of package
+def sos(srcpkg):
+    name, ver = srcpkg
+    return "src:" + name
 
 
-def sop(cudfpkg):
-    name, ver = cudfpkg
-    pref, name = name.split(':', 2)
-    if pref == 'src':
-        return "src:" + name
-    else:
-        return name
+def sob(binpkg):
+    name, arch, ver = binpkg
+    return name
+
 
 # string of list
-
-
-def sol(l):
-    return " ".join([sop(pkg) for pkg in l])
+def sobl(l):
+    return " ".join([sob(pkg) for pkg in l])
 
 if __name__ == '__main__':
     import argparse
@@ -33,4 +32,4 @@ if __name__ == '__main__':
     args = parser.parse_args()
     for scc in args.stats['buildgraph']['sccs']:
         for src, deps in scc['fas']:
-            print(sop(src), sol(deps))
+            print(sos(src), sobl(deps))
