@@ -12,49 +12,43 @@ from functools import reduce
 def partition(l, p):
     return reduce(lambda x, y: x[not p(y)].append(y) or x, l, ([], []))
 
+
 # string of package
+def sob(binpkg):
+    name, arch, ver = binpkg
+    return "<span class=\"pkg\" title=\"%s:%s&nbsp;(=&nbsp;%s)\">%s</span>" \
+           % (name, arch, ver, name)
 
 
-def sop(cudfpkg):
-    name, ver = cudfpkg
-    pref, name = name.split(':', 2)
-    fullname = "%s:%s&nbsp;(=&nbsp;%s)" % (pref, name, ver)
-    if pref == 'src':
-        return "<a href=\"/source/%s_%s.html\" class=\"pkg\"" % (name, ver) + \
-               " title=\"%s\">%s</a>" % (fullname, "src:" + name)
-    else:
-        return "<span class=\"pkg\" title=\"%s\">%s</span>" % (fullname, name)
+def sos(srcpkg):
+    name, ver = srcpkg
+    return "<a href=\"/source/%s_%s.html\" class=\"pkg\"" % (name, ver) + \
+           " title=\"src:%s&nbsp;(=&nbsp;%s)\">src:%s</a>" % (name, ver, name)
+
 
 # string of list
+def sobl(l):
+    return ", ".join([sob(pkg) for pkg in l])
 
 
-def sol(l):
-    return ", ".join([sop(pkg) for pkg in l])
+def sosl(l):
+    return ", ".join([sos(pkg) for pkg in l])
+
 
 # string of vector
-
-
 def sov(vertex):
     t, data = vertex
     if t == 'SrcPkg':
         name, ver = data
-        pref, name = name.split(':', 2)
-        fullname = "%s:%s&nbsp;(=&nbsp;%s)" % (pref, name, ver)
-        if pref != 'src':
-            raise Exception(
-                "SrcPkg vertices must have package starting with 'src:'")
+        fullname = "src:%s&nbsp;(=&nbsp;%s)" % (name, ver)
         return "<a href=\"/source/%s_%s.html\" class=\"pkg\"" % (name, ver) + \
-               " title=\"%s\">%s</a>" % (fullname, "src:" + name)
+               " title=\"%s\">src:%s</a>" % (fullname, name)
     elif t == 'InstSet':
-        (name, ver), instset = data
-        pref, name = name.split(':', 2)
-        instset = ", ".join(["%s&nbsp;(=&nbsp;%s)" % (n, v)
-                            for n, v in instset])
+        (name, arch, ver), instset = data
+        instset = ", ".join(["%s:%s&nbsp;(=&nbsp;%s)" % (n, a, v)
+                            for n, a, v in instset])
         fullname = "%s:%s&nbsp;(=&nbsp;%s), installation set = { %s }" % (
-            pref, name, ver, instset)
-        if pref == 'src':
-            raise Exception(
-                "InstSet vertices must not have package starting with 'src:'")
+            name, arch, ver, instset)
         return "<span class=\"pkg\" title=\"%s\">%s</span>" % (fullname, name)
     elif t == 'SCC':
         raise Exception("SCC not implemented yet")
@@ -89,7 +83,7 @@ tables = {
                      "binary package it itself builds</p>",
             'heads': ["source package", "strongly depends on",
                       "because of the source package build depending on"],
-            'printer': lambda src, l1, l2: (sop(src), sol(l1), sol(l2)),
+            'printer': lambda src, l1, l2: (sos(src), sobl(l1), sobl(l2)),
             'order': 0
         },
         'type2cycles': {
@@ -99,7 +93,7 @@ tables = {
                      "direct build dependencies.</p>",
             'heads': ["source package", "strongly depends on",
                       "because of the source package build depending on"],
-            'printer': lambda src, l1, l2: (sop(src), sol(l1), sol(l2)),
+            'printer': lambda src, l1, l2: (sos(src), sobl(l1), sobl(l2)),
             'order': 0
         },
         'type3cycles': {
@@ -107,7 +101,7 @@ tables = {
             'descr': "<p>Not-strong self-cycles which can probably be " +
                      "solved by choosing a different installationset</p>",
             'heads': ["source package", "in non-strong cycle with"],
-            'printer': lambda src, deps: (sop(src), sol(deps)),
+            'printer': lambda src, deps: (sos(src), sobl(deps)),
             'order': 0
         },
     },
@@ -137,7 +131,7 @@ tables = {
             "Source packages which only lack very few build dependencies.",
             'heads': ["Amount", "Source Package", "Build Dependencies"],
             'printer': lambda src, deps:
-            ("%d" % (len(deps)), sop(src), sol(deps)),
+            ("%d" % (len(deps)), sos(src), sobl(deps)),
             'order': 0
         },
         'ratio_source': {
@@ -164,8 +158,8 @@ third column by the amount of packages in the fifth column</p>""",
                       "Source Package is needed by",
                       "Those Packages are Build Dependencies of"],
             'printer': lambda s, b, n, o:
-            ("%.02f" % (float(b) / len(o)), sop(s), "%d" %
-             (b), sol(n), sol(o)),
+            ("%.02f" % (float(b) / len(o)), sos(s), "%d" %
+             (b), sobl(n), sosl(o)),
             'order': 1
         },
         'ratio_binary': {
@@ -191,19 +185,20 @@ third column by the amount of packages in the fourth column</p>""",
              "Needs Source Package",
              "Is Build Dependency of"],
             'printer': lambda b, s, o:
-            ("%.02f" % (float(s) / len(o)), sop(b), "%d" % (s), sol(o)),
+            ("%.02f" % (float(s) / len(o)), sob(b), "%d" % (s), sosl(o)),
             'order': 1
         },
         'weak': {
             'title': "Only Weak Build Dependencies Missing",
             'descr': "<p>Source packages that only lack \"weak\" " +
                      "build dependencies as classified as such by " +
-                     "<a href=\"https://gitorious.org/debian-bootstrap/botch" +
-                     "/blobs/master/droppable/weak-build-dependencies.list\"" +
+                     "<a href=\"https://gitlab.mister-muffin.de/" +
+                     "debian-bootstrap/botch" +
+                     "/blob/master/droppable/weak-build-dependencies.list\"" +
                      ">this list</a>.</p>",
             'heads': ["Amount", "Source Package", "Weak Build Dependencies"],
             'printer': lambda src, deps:
-            ("%d" % (len(deps)), sop(src), sol(deps)),
+            ("%d" % (len(deps)), sos(src), sobl(deps)),
             'order': 0
         },
         'srcbinstats': {
@@ -220,8 +215,8 @@ third column by the amount of packages in the fourth column</p>""",
              "Out-Degree",
              "Ratio",
              "Connectedness"],
-            'printer': lambda src, outd, ind:
-            (sop(src), "%d" % (ind), "%d" % (outd), "%.03f" %
+            'printer': lambda v, outd, ind:
+            (sov(v), "%d" % (ind), "%d" % (outd), "%.03f" %
              (float(ind) / float(outd)), "%d" % (ind + outd)),
             'order': 0
         },
@@ -230,7 +225,7 @@ third column by the amount of packages in the fourth column</p>""",
             'descr': "A feedback arc set which would make the dependency " +
                      "graph acyclic.",
             'heads': ["Source Package", "Build Dependencies to Drop"],
-            'printer': lambda src, deps: (sop(src), sol(deps)),
+            'printer': lambda src, deps: (sos(src), sobl(deps)),
             'order': 0
         },
         'sap': {
@@ -305,7 +300,7 @@ def render_table(table, name, dat, num):
     print("<thead><tr>")
     for head in table['heads']:
         print("<th>%s</th>" % head)
-    print("</tr></thead><tbody>")
+    print("</tr></thead>\n<tbody>")
     if dat[name]:
         for row in dat[name]:
             # we have to do this so that it works in python2 and python3
@@ -314,19 +309,23 @@ def render_table(table, name, dat, num):
             getattr(sys.stdout, 'buffer', sys.stdout).flush()
             getattr(sys.stdout, 'buffer', sys.stdout).write(
                 ('<tr><td>' + '</td><td>'.join(table['printer'](*row)) +
-                 '</td></tr>').encode("utf8"))
+                 '</td></tr>\n').encode("utf8"))
             getattr(sys.stdout, 'buffer', sys.stdout).flush()
             sys.stdout.flush()
     else:
-        print('<tr><td colspan="%d">empty</td></tr>' % len(table['heads']))
+        print('<tr><td colspan="%d">empty</td></tr>\n' % len(table['heads']))
     print("</tbody></table>")
     print("</div>")
 
 
 def stat_html(data, online=False, verbose=False):
+    # sort SCCs with descending number of vertices
+    # if the number of vertices is the same, sort by the number of edges, then
+    # by the vertex properties
     sccs = list(
         enumerate(sorted(data['buildgraph']['sccs'],
-                  key=lambda x: x['sg_vertex'], reverse=True)))
+                  key=lambda x: (x['sg_vertex'], x['sg_edges'],
+                                 x['srcbinstats']), reverse=True)))
 
     if online:
         webresources_style = "/webresources"
@@ -421,9 +420,8 @@ def stat_html(data, online=False, verbose=False):
         print("        toggle_scc(%d)" % (i + 1))
     print("""      }
       function toggle_result_all () {""")
-    # FIXME: sort this
-    for i in (list(tables['srcgraph'].keys()) +
-              list(tables['buildgraph'].keys())):
+    for i in (sorted(list(tables['srcgraph'].keys())) +
+              sorted(list(tables['buildgraph'].keys()))):
         print("        toggle_result('%s')" % i)
     print("""      }
       function hide_scc_all () {
@@ -463,9 +461,8 @@ def stat_html(data, online=False, verbose=False):
         toggle_result_all();
       }
       function restore_default_order () {""")
-    # FIXME: sort this
-    for name, val in (list(tables['srcgraph'].items()) +
-                      list(tables['buildgraph'].items())):
+    for name, val in (sorted(list(tables['srcgraph'].items())) +
+                      sorted(list(tables['buildgraph'].items()))):
         print("        $('.%s').trigger(\"sorton\",[[[0,%d]]]);"
               % (name, val['order']))
     print("""      }
@@ -485,14 +482,12 @@ def stat_html(data, online=False, verbose=False):
         $('#hidesrc').click(toggle_hidesrc);
         $('#hidebuild').click(toggle_hidebuild);""")
 
-    # FIXME: sort this
-    for i, scc in sccs:
+    for i, scc in sorted(sccs):
         print("        $('#show_scc_%d').click(function() {toggle_scc(%d);})"
               % ((i + 1), (i + 1)))
 
-    # FIXME: sort this
-    for i in (list(tables['srcgraph'].keys()) +
-              list(tables['buildgraph'].keys())):
+    for i in (sorted(list(tables['srcgraph'].keys())) +
+              sorted(list(tables['buildgraph'].keys()))):
         print("        $('#show_%s').click(function() {toggle_result('%s');})"
               % (i, i))
 
