@@ -52,15 +52,78 @@ def read_json_file(path):
 
 
 def read_graphml(path):
+    try:
+        if path == '-':
+            return nx.read_graphml(sys.stdin)
+        else:
+            return nx.read_graphml(path)
+    except:
+        import argparse
+        raise argparse.ArgumentTypeError(
+            "Input graph \"%s\" is not in GraphML format" % path)
+
+
+def read_graph(path):
+    origpath = path
     if path == '-':
-        return nx.read_graphml(sys.stdin)
+        path = sys.stdin
+    # try reading graphml first because xml.etree.ElementTree will properly
+    # fail if it cannot read the input while pygraphviz will parse XML
+    # without raising an exception
+    # monkey patching the output graph to not loose the input file type
+    try:
+        g = nx.read_graphml(path)
+        g.input_file_type = "graphml"
+    except:
+        try:
+            g = nx.read_dot(path)
+            g.input_file_type = "dot"
+        except:
+            import argparse
+            raise argparse.ArgumentTypeError(
+                "Input graph \"%s\" is neither in GraphML nor in dot format"
+                % origpath)
+    return g
+
+
+def write_graph(path):
+    if path == '-':
+        def helper(g):
+            if not hasattr(g, "input_file_type"):
+                raise Exception("this function needs the input graph to have "
+                                "the attribute input_file_type")
+            if g.input_file_type == "graphml":
+                nx.write_graphml(g, sys.stdout.buffer)
+            elif g.input_file_type == "dot":
+                nx.write_dot(g, sys.stdout.buffer)
+            else:
+                raise Exception("input_file_type attribute must either be dot "
+                                "or graphml")
     else:
-        return nx.read_graphml(path)
+        def helper(g):
+            if path.endswith(".xml"):
+                nx.write_graphml(g, path)
+            elif path.endswith(".dot"):
+                nx.write_dot(g, path)
+            else:
+                # cannot determine desired output file type from output file
+                # extension, fall back to input file type
+                if not hasattr(g, "input_file_type"):
+                    raise Exception("this function needs the input graph to "
+                                    "have the attribute input_file_type")
+                if g.input_file_type == "graphml":
+                    nx.write_graphml(g, path)
+                elif g.input_file_type == "dot":
+                    nx.write_dot(g, path)
+                else:
+                    raise Exception("input_file_type attribute must either be "
+                                    "dot or graphml")
+    return helper
 
 
 def write_graphml(path):
     if path == '-':
-        return lambda g: nx.write_graphml(g, sys.stdout)
+        return lambda g: nx.write_graphml(g, sys.stdout.buffer)
     else:
         return lambda g: nx.write_graphml(g, path)
 
@@ -196,7 +259,7 @@ def graph_remove_weak(g, weak_deps):
     def is_weak(a):
         if a['kind'] == 'SrcPkg':
             return False
-        n = a['name'].split(':')[1]
+        n = a['name']
         return n in weak_deps
     weak_inst_sets = [n for n, a in g.nodes_iter(data=True) if is_weak(a)]
     g.remove_nodes_from(weak_inst_sets)
@@ -207,8 +270,8 @@ def graph_remove_droppable(g, reduced_deps):
         v1, v2 = e
         if g.node[v1]['kind'] == 'InstSet':
             False
-        n1 = g.node[v1]['name']
-        n2 = g.node[v2]['name'].split(':')[1]
+        n1 = "src:" + g.node[v1]['name']
+        n2 = g.node[v2]['name']
         return n1 in reduced_deps and n2 in reduced_deps[n1]
     droppable_edges = [e for e in g.edges_iter() if is_droppable(e)]
     g.remove_edges_from(droppable_edges)
@@ -257,12 +320,82 @@ def arch_matches(arch, wildcard):
 
 
 def parse_dose_yaml(yamlin):
-    missing = defaultdict(lambda: defaultdict(set))
-    conflict = defaultdict(lambda: defaultdict(set))
     data = {"bin": defaultdict(set), "src": defaultdict(set)}
 
+    if yamlin.get('output-version') is None:
+        raise Exception('missing yaml field output-version')
+
+    if yamlin['output-version'] != '1.0':
+        raise Exception('yaml output-version is unequal 1.0: ' +
+                        yamlin['output-version'])
+
     if yamlin.get('report') is None:
-        return ({}, {}, {"bin": {}, "src": {}})
+        return {"bin": {}, "src": {}}
+
+    for p in yamlin['report']:
+        if p['status'] == 'broken':
+            for r in p['reasons']:
+                if r.get('missing'):
+                    data['bin'][r['missing']['pkg']['unsat-dependency']
+                                .split(' ', 1)[0].split(':',
+                                                        1)[0]].add('missing')
+                    data['src'][p['package'].split(':', 1)[0]].add('missing')
+                if r.get('conflict'):
+                    data['bin'][r['conflict']['pkg1'][
+                        'package'].split(':', 1)[0]].add('conflict')
+                    data['bin'][r['conflict']['pkg2'][
+                        'package'].split(':', 1)[0]].add('conflict')
+                    data['src'][p['package'].split(':', 1)[0]].add('conflict')
+    return data
+
+
+def vpkgdisj2deb(disj):
+    disjl = disj.split('|')
+
+    def vpkg2deb(vpkg):
+        vpkg = vpkg.strip()
+        try:
+            name, ver = vpkg.split(' ', 1)
+        except ValueError:
+            name = vpkg
+            ver = None
+        try:
+            name, arch = name.split(':', 1)
+        except ValueError:
+            arch = None
+        if name.startswith('--virtual-'):
+            name = name[10:]
+        if ver:
+            return "%s:%s %s" % (name, arch, ver)
+        else:
+            if arch:
+                return "%s:%s" % (name, arch)
+            else:
+                return name
+    # make unique, sort and join
+    return " | ".join(sorted(list(set([vpkg2deb(vpkg) for vpkg in disjl]))))
+
+
+def get_depchain(depchain):
+    firstpkg = ("", None, vpkgdisj2deb(depchain['depchain'][0]['depends']))
+    chain = [(k['package'], k['version'], vpkgdisj2deb(k['depends']))
+             for k in depchain['depchain'][1:]]
+    return [firstpkg] + chain
+
+
+def parse_dose_yaml_mc(yamlin):
+    missing = defaultdict(lambda: defaultdict(set))
+    conflict = defaultdict(lambda: defaultdict(set))
+
+    if yamlin.get('output-version') is None:
+        raise Exception('missing yaml field output-version')
+
+    if yamlin['output-version'] != '1.0':
+        raise Exception('yaml output-version is unequal 1.0: ' +
+                        yamlin['output-version'])
+
+    if yamlin.get('report') is None:
+        return ({}, {})
 
     for p in yamlin['report']:
         n, v = p['package'], p['version']
@@ -271,63 +404,41 @@ def parse_dose_yaml(yamlin):
         if p['status'] == 'broken':
             for r in p['reasons']:
                 if r.get('missing'):
-                    unsatdep = (r['missing']['pkg']['unsat-dependency']
-                                .split(' ', 1)[0], None)
+                    pkg = r['missing']['pkg']
+                    unsatdep = vpkgdisj2deb(pkg['unsat-dependency'])
+                    lastpkg = (pkg['package'], pkg['version'], unsatdep)
                     c = r['missing'].get('depchains')
                     if c:
+                        depchains = [tuple(get_depchain(depchain) + [lastpkg])
+                                     for depchain in c]
                         # normalize the order of depchains
-                        depchains = tuple(sorted(set(
-                            [tuple([("", None)] + [(p['package'], p['version'])
-                             for p in depchain['depchain'][1:]] + [unsatdep])
-                             for depchain in c])))
+                        depchains = tuple(sorted(set(depchains)))
                     else:
-                        depchains = tuple([(("", None), (unsatdep))])
-                    missing[unsatdep][depchains].add((n, v))
-                    data['bin'][r['missing']['pkg']['unsat-dependency']
-                                .split(' ', 1)[0].split(':',
-                                                        1)[1]].add('missing')
-                    data['src'][n.split(':', 1)[1]].add('missing')
+                        depchains = tuple([(("", None, unsatdep),)])
+                    missing[unsatdep][depchains].add((n, v, None))
                 if r.get('conflict'):
-                    pkg1 = (r['conflict']['pkg1']['package'],
-                            r['conflict']['pkg1']['version'])
-                    pkg2 = (r['conflict']['pkg2']['package'],
-                            r['conflict']['pkg2']['version'])
+                    conf1 = r['conflict']['pkg1']
+                    conf2 = r['conflict']['pkg2']
+                    unsatconf = vpkgdisj2deb(conf1['unsat-conflict'])
+                    lastpkg1 = (conf1['package'], conf1['version'], unsatconf)
+                    lastpkg2 = (conf2['package'], conf2['version'], None)
                     c1 = r['conflict'].get('depchain1')
                     if c1:
-                        depchain1 = tuple(sorted(set(
-                            [tuple([("", None)] + [(p['package'], p['version'])
-                             for p in depchain['depchain'][1:]] + [pkg1])
-                             for depchain in c1])))
+                        depchain1 = [tuple(get_depchain(depchain) + [lastpkg1])
+                                     for depchain in c1]
+                        depchain1 = tuple(sorted(set(depchain1)))
                     else:
-                        depchain1 = tuple([(("", None), pkg1)])
+                        depchain1 = tuple([(("", None, unsatconf),)])
                     c2 = r['conflict'].get('depchain2')
                     if c2:
-                        depchain2 = tuple(sorted(set(
-                            [tuple([("", None)] + [(p['package'], p['version'])
-                             for p in depchain['depchain'][1:]] + [pkg2])
-                             for depchain in c2])))
+                        depchain2 = [tuple(get_depchain(depchain) + [lastpkg2])
+                                     for depchain in c2]
+                        depchain2 = tuple(sorted(set(depchain2)))
                     else:
-                        depchain2 = tuple([(("", None), pkg2)])
-                    # a conflict with depchains [A, B] is equal to a conflict
-                    # with depchains [B, A], so normalize the order
-                    if depchain1 is None and depchain2 is None:
-                        depchains = (None, None)
-                    elif depchain1 is None:
-                        depchains = (depchain2, None)
-                    elif depchain2 is None:
-                        depchains = (depchain1, None)
-                    else:
-                        depchains = tuple(sorted([depchain1, depchain2]))
-                    # conflict A<->B is equal to a conflict B<->A, so normalize
-                    # the order
-                    conflict[tuple(sorted([pkg1, pkg2]))][depchains].add((n,
-                                                                          v))
-                    data['bin'][r['conflict']['pkg1'][
-                        'package'].split(':', 1)[1]].add('conflict')
-                    data['bin'][r['conflict']['pkg2'][
-                        'package'].split(':', 1)[1]].add('conflict')
-                    data['src'][n.split(':', 1)[1]].add('conflict')
-    return (missing, conflict, data)
+                        depchain2 = tuple([(("", None, None),)])
+                    depchains = tuple([depchain1, depchain2])
+                    conflict[unsatconf][depchains].add((n, v, None))
+    return (missing, conflict)
 
 
 def human_readable_size(val):
@@ -336,3 +447,22 @@ def human_readable_size(val):
             return "%.1f %s" % (val, unit)
         val /= 1024.0
     return "%.1f GiB" % val
+
+
+def find_node(g, selectors):
+    packages = find_nodes(g, selectors)
+    if len(packages) > 1:
+        sel = ",".join(["%s:%s" % s for s in selectors])
+        print("found %d results for selectors %s - picking first"
+              % (len(packages), sel), file=sys.stderr)
+    return packages[0]
+
+
+def find_nodes(g, selectors):
+    packages = [n for n, attr in g.nodes(data=True)
+                if all([attr.get(k) == v if k != '__ID__' else n == v
+                        for k, v in selectors])]
+    if len(packages) == 0:
+        sel = ",".join(["%s:%s" % s for s in selectors])
+        raise Exception("package cannot be found for selectors %s" % sel)
+    return sorted(packages)
